(function () {
	Vue.filter('joborder', function () {
		return function (input) {
			if (!input) {
				return;
			}

			return '#' + input.id + ' ' + input.jobTitle + (input.client ? ' of ' + input.client.name : '') + (input.bu ? ' of '+  input.bu.name : '');
		};
	});
	Vue.component('ui-select', {
		template: `
			<div>
			    <input type="text" class="comtree span3 comtree-offscreen"/>
			    <div class="comtree span3" :class="{on: dropdowned}">
			        <a href="#" class="comtree-title" @click.prevent="dropdowned = true">
			            <span :title="ititle">{{ititle}}</span>
			            <b></b>
			            <i class="fn-remove-all" @click.prevent.stop="remove"/>
			        </a>
			        <div class="comtree-drop-mask" @click="dropdowned = false"></div>
			        <div class="comtree-drop">
			            <div class="comtree-drop-inner">
			                <div class="search">
			                    <input type="text" class="fn-search-input form-control" v-model="searchWord" autocomplate="false"/>
			                </div>
			                <div class="comtree-tree">
			                    <ul class="jqtree_common jqtree-tree">
			                    <li class="jqtree_common" v-for="option in options" v-show="search(option.value)">
			                        <div class="jqtree-element jqtree_common" @click.stop="choose(option)">
			                        	<span v-bind="option | joborder"></span>
			                        </div>
			                    </li>
			                    </li>
			                </div>
			                <div class="comtree-button clearfix">
			                    <button 
			                        class="btn btn-small btn-default btn-sm pull-right fn-confirm"
			                        @click.prevent="dropdowned = false"
			                    >新增</button>
			                </div>
			            </div>
			        </div>
			    </div>
			</div>
		`,
		data: function(){
			return {
				dropdowned: false,
				searchWord: '',
				options: [],
				indexOption: ''

			};
		},
		computed: {
			ititle: function(){
				var me = this;
				if(!me.indexOption){
					return ''
				}
				return '#' + me.indexOption.id + ' ' + me.indexOption.jobTitle + (me.indexOption.client ? ' of ' + me.indexOption.client.name : '') + (me.indexOption.bu ? ' of '+  me.indexOption.bu.name : '');
			},
			indexOptionId: function () {
				var me = this;
				return me.indexOption.id;
			}
		},
		create: function () {
			var me = this;
			var gql = {};
			var data = {
				ordering: '-lastUpdateDate',
				paginate_by: '50',
				demandKeys: JSON.stringify(["bu", "lineManager"])
			};

			if (jobTitle) {
				var gql = '';
				if (parseInt(jobTitle, 10)) {
					gql = 'id='+jobTitle+'|jobTitle__icontains='+jobTitle;
				} else {
					gql = 'jobTitle__icontains='+jobTitle;
				}
				data['gql'] = gql;
			} else {
				data['gql'] = $.param({
					jobStatus: 'Live',
					joborderuser_set__user__eq: '{{user.id}}'
				});
			}
			return Utils.getJSON('/rest/user/context').then(function(context) {
				return Utils.getJSON('/rest/joborder/list', data).then(function (res) {
					me.options = res.list;
				});
			});
		},
		watch: {
			searchWord: {
				handler: function (val, oldVal) {
					var me = this;
					function debounce(fn, delay) {
						var timer = null;
						return function () {
							var context = this, args = arguments;
							clearTimeout(timer);
							timer = setTimeout(function () {
								fn.apply(context, args);
							}, delay);
						};
					}
					debounce(function () {
						me.refreshJoborders(val);
					}, 250)();
				}
			}
		},
		methods: {
			search: function(v){
				return v.toLowerCase().indexOf(this.searchWord.toLowerCase()) > -1;
			},
			choose: function(opt){
				var me = this;
				me.indexOption = opt;
				this.$emit('input', this.indexOption);
			},
			remove: function(){
				this.indexOption = '';
				this.$emit('input', []);
			},
			refreshJoborders: function (event) {
				var me = this;
				var gql = {};
				var data = {
					ordering: '-lastUpdateDate',
					paginate_by: '50',
					demandKeys: JSON.stringify(["bu", "lineManager"])
				};
				var jobTitle = event.target.innerText;
				if (jobTitle) {
					var gql = '';
					if (parseInt(jobTitle, 10)) {
						gql = 'id='+jobTitle+'|jobTitle__icontains='+jobTitle;
					} else {
						gql = 'jobTitle__icontains='+jobTitle;
					}
					data['gql'] = gql;
				} else {
					data['gql'] = $.param({
						jobStatus: 'Live',
						joborderuser_set__user__eq: '{{user.id}}'
					});
				}
				return Utils.getJSON('/rest/user/context').then(function(context) {
					return Utils.getJSON('/rest/joborder/list', data).then(function (res) {
						me.options = res.list;
					});
				});
			}
		}
	})
})()