/* global chrome, Cookie, Ajax, Logger, Runtime, gStorage, Tab, WebRequest, Config, Utils, saas*/

/**
 *
 */
(function(root, ajax) {
    'use strict';
    root.Posting = root.Posting || {};

    var performance = window.performance;

    var Utils = root.Utils;

    // Set queue timer to 1 mins by default
    var timerTicks = 1000 * 60 * 1;


    /**
     * Config the match regex
     */
    function buildMatcher() {
        Config.REG_URL = new RegExp(Config.HOST);

        Logger.log('config REG_URL: ', Config.REG_URL);
    }

    function afterMilliseconds(delay){
        return function(res){
            return new Promise(function(resolve, reject){
                setTimeout(function(){
                    resolve(res)
                }, delay);
            });
        }
    }

    function removeTab(tabId) {
        chrome.tabs.remove(tabId, function() {});
    }


    /**
     * Process extra data string to JSON before send request to target channel
     * @param data
     * @returns {*|{}}
     */
    function processData(data) {
        var regDomain = /http[s]?:\/\/([^\/])+/;

        data = data || {};

        Logger.log(data.extra_data);

        try {
            data.extra_data = data.extra_data ? $.parseJSON(data.extra_data) : {};

            data.form = data.form ? $.parseJSON(data.form) : {};
        } catch (e) {}
        var cookies = data.extra_data.cookies || data.cookies;

        if (cookies) {
            Cookie.setCookies($.extend({}, {
                url: data.url,
                domain: Config.getDomainByURL(data.url)
            }, cookies));
        }

        return data;
    }

    /**
     * Refine request data
     * @param data
     * @returns {*}
     */
    function clearNoUsedFields(data) {
        var noused = ['url', 'method', 'form'];

        if(data) {
            noused.forEach(function(field) {
                delete data[field];
            });
        }

        return data;
    }

    /**
     * Update request header before send request
     * - Set headers given by backend
     * - remove 'Referer' header if backend not provided
     * - remove 'Origin' header if backend not provided
     *
     * @param queue
     *
     */
    function checkAndUpdateHeader(queue) {
        var headers;
        try{
            headers = JSON.parse(queue.headers) || {};
        } catch(err){
            headers = {};
        }


        WebRequest.addHeaderHandlerOnce(function(details, curr) {
            if(Utils.getHostByURL(queue.url) !==
                Utils.getHostByURL(details.url)) {
                return curr;
            }

            Logger.log('[Web Request Header Updated] info:%o',
                queue.headers);

            Object.keys(headers).forEach(function(name) {
                curr[name] = headers[name];
            });

            // remove all referer and origin header for every request by default
            if(!headers['Referer']) {
                delete curr['Referer'];
            }

            if(!headers['Origin']) {
                delete curr['Origin'];
            }

            return curr;
        });

    }

    /**
     *
     * @param res
     */
    function checkSystemLoginStatus(res) {
        if (res.message === "login required") {
            Logger.warn("Gllue login expired!");
            return false;
        }

        return true;
    }

    // Job Posting App
    var App = {
        /*
        *  Posting Extension init
        *  1. save plugin info to storage
        *  2. bind tab listener to listen all tab updates (create/refresh/close etc.)
        *  3. init all events
        *  4. run queue
        *  5. set posting status to 'inited'
        * */
        init: function onInit() {

            // Store gllue extension info for Iframe communicate
            gStorage.set({
                "gllue-pinfo":  Runtime.getExtensionInfo()
            });

            Tab.onUpdate(function(tabId, changeInfo, tab) {
                var url = tab.url;
                var match = url.match(Config.REG_URL);

                if (Config.HOST && match) {
                    console.log('write tabId to tab: ', tab.id);
                    Utils.writeTabIdToWebPage();
                }
            });

            this.initEvents();

            // when url is starts with 'https'
            if(Config.HOST && Config.HOST.indexOf('https') === 0) {
                Tab.create({
                    url: Config.HOST,
                    cb: function(tab) {
                        Logger.warn('Url starts with "https", queue stopped.');
                    }
                });
            // when url is start with 'http'
            } else {
                // ask queue
                scheduleQueue();
            }
            this.inited = true;
        },
        runSession: runSession,

        initEvents: function() {
            var me = App;

            // Content page refreshed
            Runtime.onExtMessage("refresh", function() {
                Tab.getActiveTab(function(tab) {
                    // loadedSite[tab] = {};
                });
            });

            // verfiy code input
            var evts = ['verify'].map(function(item) {
                return Config.DOMAIN.map(function(site) {
                    return  item + '/' + site;
                }) + '';
            }) + '';

            //Runtime.onExtMessage(evts.toLowerCase(), function onLoginOrVerify(request, sender) {
            //    runSession(request.session_id, {
            //        result: request.value,
            //        uuid: request.uuid
            //    }).then(function(res){
            //        notifyAndResolve(sender.tab.id, 'loading', res);
            //    });
            //
            //});
        }
    };

    var runningSessionList = {};


    var queueTimer;

    function scheduleQueue(){
        Logger.log("[Schedule Queue]");
        function runActiveSessions(){
            ajax({
                url: Config.HOST + '/rest/jobposting/active_sessions'
            }).then(function(res){
                if (!res.status && res.message){
                    Logger.log("[Run Active Sessions Failed]" + res.message);
                }
                // 没有active_session
                if (!res.data || !res.data.session_ids){
                    return;
                }
                res.data.session_ids.forEach(function(sessionId){
                    // runSession保证同一个session只run一次
                    runSession(sessionId);
                });
            });
        }
        queueTimer = window.setInterval(runActiveSessions, timerTicks);
        runActiveSessions();
        window.runActiveSessions = runActiveSessions;
    }

    function stopScheduleQueue(){
        clearInterval(queueTimer);
    }



    function runSession(session_id, data){
        data = data || {};
        data.session_id = session_id;
        var p = getRunningSession(session_id);
        if (!p){
            var runSingleQueue = runSingleQueueGenerator(session_id);
            p = loopUntil(function(res){
                return res.__session_done;
            }, runSingleQueue(data), function(res) {
                return runSingleQueue(res);
            }).then(function(ret){
                ret = ret || {};
                return ret;
            });
            addRunningSession(session_id, p);
        }
        return p.then(function(ret){
            removeRunningSession(session_id);
            return ret;
        })
    }

    function loopUntil(condition, start, body){
        return start.then(function(res){
            if (condition(res)){
                return res;
            }else{
                return loopUntil(condition, body(res), body)
            }
        })
    }

    function getRunningSession(session_id){
        return runningSessionList[session_id];
    }

    function addRunningSession(session_id, p){
        runningSessionList[session_id] = p;
    }

    function removeRunningSession(session_id){
        if (runningSessionList[session_id]){
            delete runningSessionList[session_id];
        }
    }


    function runSingleQueueGenerator(session_id){
        return function runSignleQueue(data){
            data = data || {};
            data.session_id = session_id;
            data = clearNoUsedFields(data);
            if (Utils.isObject(data) && data.result) {
                data.result = Utils.deflate(data.result);
            }
            return ajax({
                url: Config.HOST + '/rest/jobposting/queue',
                type: 'post',
                data: (data || '')
            }).then(function(res){
                if (!checkSystemLoginStatus(res)) {
                    // 系统未登录
                    return afterMilliseconds(5 * 60 * 1000)({});
                } if (!res.next_queue && !res.last_queue){
                    // 出错了, blindly waiting
                    Logger.error("跑ression时获取last_queue失败."+JSON.stringify(res));
                    return afterMilliseconds(20 * 1000)({});
                } if (res.next_queue && res.next_queue.pending) {
                    // pendingQueue，直接返回
                    return afterMilliseconds((res.next_queue.sleep_time || 1) * 1000)({
                        session_id: res.session,
                        pending_queue_response: true
                    });
                }

                var session = res.session;
                var next_queue = res.next_queue;
                var last_queue = res.last_queue;
                last_queue = last_queue || {};
                if (!session.done && !session.sleep && next_queue){
                    // 正常情况
                    checkAndUpdateHeader(next_queue);
                    return sendRequest(next_queue);
                } else if (session.sleep) {
                    // session sleep
                    return afterMilliseconds(3 * 60 * 1000)({});
                } else {
                    // session done
                    last_queue.__session_done = true;
                    return last_queue;
                }
            }, function(err){

            });

        };
    }


    function sendRequest(data){
        var redirectUrls = {};

        function redirectListener(headers) {
            var statusCode = headers.statusCode;
            var locationUrl;
            if (statusCode === 302 || statusCode === 304 || statusCode === 301) {
                headers.responseHeaders.forEach(function (item) {
                    if (item.name == "Location") {
                        locationUrl = item.value;
                    }
                });
                if (locationUrl) {
                    redirectUrls[headers.url] = locationUrl;
                }
            }
        }

        chrome.webRequest.onHeadersReceived.removeListener(redirectListener);
        chrome.webRequest.onHeadersReceived.addListener(redirectListener, {
            urls: [
                "*://*.zhaopin.com/*"
            ]
        }, ["blocking", "responseHeaders"]);

        var requestPromise = new Promise(function(resolve, reject){
            var startTime;
            var requestHeaders;

            data = processData(data) || {};


            function _resolve(res, statusCode, headers) {
                var endTime = performance.now();
                data.result = res;
                // set response status code
                data.statusCode = statusCode;
                // set response headers
                data.headers = JSON.stringify(headers);
                // set request response time
                data.excuteTime = endTime - startTime;
                // set request headers
                data.requestHeaders = requestHeaders;

                data.redirectUrls = JSON.stringify(redirectUrls);

                chrome.webRequest.onHeadersReceived.removeListener(redirectListener);

                var p = Promise.resolve(data);

                if (statusCode / 100 === 2){
                    if (data.url){
                        p = Cookie.getCookieByDomain(data.url).then(function(cookies){
                            data.cookies = JSON.stringify(cookies);
                            return data;
                        });
                    }
                } else {
                    Logger.error();
                }

                p.then(resolve);
            }

            var tabTimer;
            var openedTabId;
            var tabLoadStatus = false;

            function tabLoadFinishedListener(tabId , info) {
                if (info.status == "complete") {
                    tabLoadStatus = true;
                }
            }

            function waitForTabLoadFinished () {
                if (tabLoadStatus === true) {
                    if (openedTabId) {
                        removeTab(openedTabId);
                    }
                    clearInterval(tabTimer);
                    chrome.tabs.onUpdated.removeListener(tabLoadFinishedListener);
                    _resolve('', 200, {});
                }
            }

            if (['mobile', 'email', 'avatar', 'image'].indexOf(data.extra_data) > -1){
                Utils.convertImgToBase64(data.url).then(function(arr){
                    _resolve.apply(null, arr);
                }, function(arr){
                    _resolve.apply(null, arr);
                });
            } else if (data.extra_data == 'openTab') {
                var createProperties = {
                    index: 0,
                    url: data.url,
                    selected: false,
                    pinned: false
                };

                chrome.tabs.onUpdated.removeListener(tabLoadFinishedListener);
                chrome.tabs.onUpdated.addListener(tabLoadFinishedListener);
                chrome.tabs.create(createProperties, function (tab) {
                    openedTabId = tab.id;
                });
                tabTimer = setInterval(waitForTabLoadFinished, 1000);
            } else {
                $.ajax({
                    url: data.url,
                    data: data.form,
                    type: data.method,
                    dataType: 'text',
                    beforeSend: function () {
                        startTime = performance.now();
                    }
                }).done(function (res, textStatus, jqXHR) {
                    var statusCode = jqXHR.status;
                    var headers = jqXHR.getAllResponseHeaders();
                    _resolve(res, statusCode, headers);
                }).fail(function (jqXHR, textStatus, errorThrown) {
                    var statusCode = jqXHR.status;
                    var headers = jqXHR.getAllResponseHeaders();
                    _resolve(jqXHR.responseText, statusCode, headers);
                });
            }

        });

        var sleepTime = data.sleep_time;
        if (sleepTime){
            sleepTime *= 1000;
            requestPromise = requestPromise.then(afterMilliseconds(sleepTime));
        }
        return requestPromise;
    }



    function onGetHostFromStorage(host) {
        Config.HOST = host;

        if (!Config.HOST) {
            return;
        }

        buildMatcher();
        if (!App.inited){
            App.init();
        }
    }

    // Everytime when 'Config.HOST' updated.
    gStorage.onHostChange(onGetHostFromStorage);

    // try to get the stored 'gllueHost' from storage
    gStorage.getHost().then(onGetHostFromStorage);

    // Expose specific API for external debug
    Posting.stopQueue = stopScheduleQueue;
    Posting.runSession = runSession


})(window, window.ajax);
