/* global WebRequest, chrome, Config, jQuery, gStorage, Cookie, Posting, Runtime, Logger, Utils, Tab, saas, Tracker */

(function(root, $) {
    "use strict";

    // internal variable
    var requestData = {};

    // check form fields contains or not
    function verify(form, track_session) {
        var MUST_HAVE = ['site', 'page_url', 'post_url', 'cookies'];

        for(var i = 0, len = MUST_HAVE.length; i < len; i++) {
            if(!form[MUST_HAVE[i]]) {
                Logger.warn('[Position Exception] param', MUST_HAVE[i] + ' not exists', form);
                track_session.save_position_error('表单数据缺少参数', {
                    missing_key: MUST_HAVE[i],
                    form: form
                });
                return false;
            }
        }

        return true;
    }
    var positionInfo = null;

    Runtime.onMessage('position/info', function(request, sender, sendResponse) {
        var data = request.data;

        data.html = Utils.deflate(data.html);

        positionInfo = data;
    });


    function updateRequestData(requestId, data) {
        if (!requestData[requestId]) {
            requestData[requestId] = {};
        }
        requestData[requestId] = $.extend(requestData[requestId], data);
    }

    function savePosition(detail, isResponse) {
        if (!positionInfo){
            return;
        }
        var track_session = Tracker.start_save_position_session(positionInfo);

        var requestId = detail.requestId;
        var req = requestData[requestId];
        if (!req){
            track_session.save_position_error('未获取到requestData', {
                request: detail
            });
            return;
        }
        delete requestData[requestId];

        req = $.extend(req, positionInfo);
        if (root._gl_username){
            req.username = root._gl_username;
        }
        positionInfo = null;

        if (isResponse){
            req.response = {
                statusCode: detail.statusCode,
                responseHeaders: detail.responseHeaders.reduce(function(c, i) {c[i.name] = i.value; return c;}, {})
            };


        }


        Cookie.getCookieByURL(req.url).then(function(cookie) {
            req.cookies = cookie;
            var data = req;
            if (!data.form || !data.html) {
                track_session.save_position_error('未获取到requst表单信息', {
                    requestFormData: data
                });
                return;
            }
            if (!saas.Config) {
                track_session.save_position_error('SaaS Config未配置');
                return;
            }
            if (!verify(data)) {
                track_session.save_position_error('未获取到requst表单信息', {
                    requestFormData: data
                });
                return;
            }

            console.log('[Save Position]', data);

            gStorage.getHost().then(function(host) {
                var kwargs = {
                    type: 'post',
                    url: host + saas.Config.gllueUrls.position_save,
                    cache: false,
                    processData: false,
                    contentType: false
                };
                var formData = new FormData();
                if (data.isRaw) {
                    data.form = String.fromCharCode.apply(null, new Uint8Array(data.form));
                }
                formData.append('data', JSON.stringify(data));
                kwargs.data = formData;


                $.ajax(kwargs).always(function(res) {
                    if (!res.status){
                        track_session.save_position_error("后台获取session结果失败", {
                            message: res.message
                        });
                        return;
                    }
                    Logger.log('[Position Save] ', res.message);
                    if(res.data && res.data.session_ids) {
                        var session_id = res.data.session_ids[0];
                        track_session.save_position_info("开始run session", {
                            session_id: session_id
                        });
                        Posting.runSession(session_id, res.data).then(function(res){
                            track_session.save_position_info("插件session跑完", {
                                session_id: session_id
                            });

                            ClientStubPublish.webRefreshJoborderList('<all_urls>');
                        });
                    }

                });
            });

        });



    }

    function matchUrl(url) {
        if (!saas.Config || !saas.Config.sitePositionUrlRegs) {
            return false;
        }
        var reg, exp;
        for (var i = 0, l = saas.Config.sitePositionUrlRegs.length; i < l; i++) {
            reg = saas.Config.sitePositionUrlRegs[i];
            exp = new RegExp(reg);
            if (exp.test(url)) {
                return true;
            }
        }

        return false;
    }
    // when request body sending, this happened after position/info message.
    WebRequest.addBeforeRequestHandler(function(details) {
        var post_url = details.url;
        var body, isRaw;
        var formData = (details.requestBody || {}).formData;

        if (!formData && (details.requestBody || {}).raw) {
            formData = details.requestBody.raw[0].bytes;
            isRaw = true;
        }

        if (!formData || !matchUrl(details.url)) {
            return;
        }
        // 将 数组形式的 form 转换为 字符串。
        if (!isRaw) {
            formData = Object.keys(formData)
                .reduce(function(curr, key) {
                    curr[key] = formData[key] + '';
                    return curr;
                }, {});
        }

        updateRequestData(details.requestId, {
            form: formData,
            isRaw: isRaw,
            post_url: post_url
        });

        if ((root.BLOCK_ALL_WEBREQUEST || root.BLOCK_SITE_WEBREQUEST) && positionInfo) {
            try{
                savePosition(details);
            } catch (err){
                console.error(err);
            }
        }
        if (root.BLOCK_SITE_WEBREQUEST) {
            Logger.warn("URL BLOCKED: "+details.url);
            return {cancel: true};
        }
    });

    // when request headers recieved
    WebRequest.addHeaderHandler(function(details, curr) {
        var obj = {
            headers: details.requestHeaders
        };
        if (matchUrl(details.url)) {
            updateRequestData(details.requestId, obj);
        }
        return curr;
    });

    // when response header return
    WebRequest.addReponseHandler(function(details) {
        // only process position save request
        if (!matchUrl(details.url)) {
            return;
        }
        var reqId = details.requestId;
        console.log('responseHeader: ', reqId, details);
        savePosition(details, true);
    });

})(window, jQuery);
