/* global chrome, Runtime */

(function(root, $){
    'use strict';
    root.SearchRequests = root.SearchRequests || {};
    var WebRequest = root.WebRequest;
    var saas = root.saas;
    function matchSearchUrl(url) {
        if (!saas.Config || !saas.Config.siteSearchUrlRegs) {
            return false;
        }
        var exp;
        return saas.Config.siteSearchUrlRegs.some(function(reg){
            exp = new RegExp(reg);
            return exp.test(url);
        });
    }

    WebRequest.addBeforeRequestHandler(function(details, curr){
        var search_url = details.url;
        if (!matchSearchUrl(search_url)){
            return curr;
        }
        var formData = (details.requestBody || {}).formData;
        var method = details.method;
        root.SearchRequests[details.tabId] = {
            url : search_url,
            formData : formData,
            method : method
        };
        return curr;
    });

    function getCurrentRequest(callback){
        chrome.tabs.query({
            currentWindow: true,
            active: true
        },function (tabArray) {
            var tab = tabArray[0];
            if (tab) {
                var tabId = tabArray[0].id;
                var requestInfo = root.SearchRequests[tabId];
                if (callback) {
                    callback(requestInfo);
                }
            }
        });
    }

    Runtime.onMessage('getCurrentRequest', function(request, sender, callback){
        getCurrentRequest(callback);
    });


}(window, jQuery));
