/* global Ajax, chrome, jQuery, signals, Utils */

/*
 * Ajax request life cycle manager
 */
(function(root, $){
    "use strict";

    var Signal = signals.Signal;
    var $doc = $(document);
    var SLICE = [].slice;
    var pool = [];
    var EXT_VERSION = chrome.app.getDetails().version;
    var Config = root.Config;
    var startTime;
    var currURL;

    root.extVersion = chrome.app.getDetails().version;

    function ajax(options){
        return new Promise(function(resolve, reject){
            $.ajax(options).done(resolve).fail(reject);
        });
    }

    var Ajax = {
        // before ajax request send
        beforeStart: new Signal(),
        // ajax request started to send
        started : new Signal(),
        // manaully abort ajax
        abort: new Signal(),
        // a. when Ajax requests complete with an error
        // b. manaually stopped
        aborted : new Signal(),
        // when all Ajax requests have completed
        stopped: new Signal(),
        // when ever an ajax completed
        completed: new Signal()
    };

    Ajax.abort.add(function(opt) {
        pool.forEach(function(xhr) {
            if (typeof(opt) === 'function') {
                if(!opt(xhr)) {
                    return;
                }
            }
            xhr.abort();
        });

        Ajax.stopped.dispatch(opt);
    });

    function buildQueryString(url) {
        if(url.indexOf(Config.URL.posting_queue) === -1) {
            return url;
        }

        if(Utils.isUrlContainsQueryParam(url)) {
            return url + '&ext_version=' + EXT_VERSION;
        } else {
            return url + '?ext_version=' + EXT_VERSION;
        }
    }

    $.ajaxSetup({
        beforeSend: function(jqXHR, option) {
            jqXHR.url = option.url;
            pool.push(jqXHR);

            currURL = option.url = buildQueryString(option.url);

            startTime = new Date();
        },
        complete: function(jqXHR) {
            var index = pool.indexOf(jqXHR);

            if (index > -1) {
                pool.splice(index, 1);
            }
        }
    });

    [
        {
            action: 'ajaxStart',
            signal: 'started'
        },
        {
            action: 'ajaxSend',
            signal: 'beforeStart'
        },
        {
            action: 'ajaxStop',
            signal: 'stopped'
        },
        {
            action: 'ajaxError',
            signal: 'aborted'
        },
        {
            action: 'ajaxComplete',
            signal: 'completed'
        }
    ].forEach(function(opt) {
            $doc[opt.action](function() {
                var args = SLICE.call(0);

                //Logger.log('[AJAX] action: %s', opt.action);
                Ajax[opt.signal].dispatch.apply(null, args);
            });
        });

    root.Ajax = Ajax;
    root.ajax = ajax;

})(window, jQuery);
