(function(root){

    function sendUntilACK(tabId, options, retryTime, delay, resolve, reject){
        chrome.tabs.sendMessage(tabId, options, function(response){
            if (response && response.ack){
                resolve(response.ret);
            }else if (retryTime >= 0) {
                setTimeout(function(){
                    console.log('resending.....');
                    sendUntilACK(tabId, options, retryTime - 1, delay * 1.45, resolve, reject)
                }, delay);
            } else {
                reject('Timeout sending message to content script!');
            }
        });
    }

    function proxyMethod(methodName){
        return function(){
            var argumentArr = Array.from(arguments);
            var tabId = argumentArr.shift();
            var msg = {
                name: 'gllue-ext-function-call',
                msg: {
                    methodName: methodName,
                    args: JSON.stringify(argumentArr)
                }
            };

            // 由于可能发送信息的时候,接收方的方法未注入,所以要重传直到
            // 确保可以收到 ACK
            return new Promise(function(resolve, reject){
                sendUntilACK(tabId, msg, 10, 600, resolve, reject);
            });
        }
    }

    function proxyPublish(methodName){
        return function(){
            var argumentArr = Array.from(arguments);
            var tabFilter = argumentArr.shift();
            var tabFilterOptions = null;
            if (typeof tabFilter === 'string'){
                tabFilterOptions = {
                    url: tabFilter
                };
            }else{
                tabFilterOptions = tabFilter
            }
            var msg = {
                name: 'gllue-ext-function-call',
                publish: true,
                msg: {
                    methodName: methodName,
                    args: JSON.stringify(argumentArr)
                }
            };
            return new Promise(function(resolve, reject){
                chrome.tabs.query(tabFilterOptions, function(tabs){
                    var tabIds = tabs.map(function(tab){
                        var tabId = tab.id;
                        chrome.tabs.sendMessage(tabId, msg);
                        return tabId;
                    });
                    resolve(tabIds);
                });
            });
        }

    }
    var normalMethods = [
        'sendResult',
        'changePostingModalJoborderId',
        'getPositionInfo'
    ];

    var publishMethods = [
        'webRefreshJoborderList'
    ];

    root.ClientStub = {};
    root.ClientStubPublish = {};

    normalMethods.forEach(function(name){
        root.ClientStub[name] = proxyMethod(name);
    });

    publishMethods.forEach(function(name){
        root.ClientStubPublish[name] = proxyPublish(name);
    })

})(window);