/* global Cookie, chrome, Cookie, Logger, Config, gStorage, Utils */

/*
 *  This module is used to CURD chrome.cookie
 *  Dependency:
    - Config
    - logger
 */
(function(root) {
    'use strict';

    var handlers = [];
    var cookie = chrome.cookies;

    function getCookies(buildOptions){
        return function(arg){
            return new Promise(function(resolve, reject){
                var options = buildOptions(arg);
                if (!options){
                    resolve({});
                }
                chrome.cookies.getAll(options, function(cookies){
                    cookies = cookies || [];
                    var c = {};
                    cookies.map(function(cookie) {
                        var domain = cookie["domain"];
                        var c_item = c[domain];
                        if (!c_item) {
                            c[domain] = c_item = {};
                        }
                        c_item[cookie["name"]] = cookie["value"];
                    });
                    resolve(c);
                });
            });
        }
    }
    var getCookieByURL = getCookies(function(url){
        return {
            url: url
        }
    });

    var getCookieByDomain = getCookies(function(url){
        var domainMatch = Utils.getHostNameByURL(url).match(/\w+\.(.+)/) || [];
        if (domainMatch[1]){
            return {
                domain: domainMatch[1]
            }
        }
    });

    var getCookieByHost = getCookies(function(host){
        return {
            domain: host
        }
    });


    function onChanged(cb) {
        handlers.push(cb);
    }

    function handler(cookie) {
        Logger.log(cookie, chrome.runtime.lastError);
    }

    function setCookie(details) {
        chrome.cookies.set(details, handler);
    }

    function setCookies(details) {
        var cookies = details.cookies;

        Object.keys(cookies).forEach(function(key) {
            setCookie({
                name: key,
                value: cookies[key],
                url: details.url,
                domain: details.domain,
                path: details.path,
                httpOnly: true,
                expirationDate: details.expirationDate
            });
        });
    }

    // Every time when cookie changed will recheck all domain login status
    cookie.onChanged.addListener(function(changeInfo) {
        handlers.forEach(function(handler) {
            if (typeof handler === "function") {
                handler(changeInfo);
            }
        });
    });

    function getATSCookie() {
        return gStorage.getHost().then(function(atsURL){
            return getCookieByURL(atsURL);
        });
    }

    root.Cookie = {
        onChanged: onChanged,
        getCookieByURL: getCookieByURL,
        getCookieByDomain: getCookieByDomain,
        setCookies: setCookies,
        setCookie: setCookie,
        getCookieByHost: getCookieByHost,
        getATSCookie: getATSCookie
    };

})(window);
