(function(root) {
    "use strict";

    // CONSTANT
    var isLegacy = 'webkitNotification' in window;
    var GRUNTED = "granted";

    // Build compatible for old 'webkitNotification' and 'Notifcation'
    var Notify = isLegacy ? root.webkitNotification : root.Notification;

    var DEFAULT_SETTING = {
        body: '',
        icon: 'images/extension_icon_large.png',
        tab: 'gllue-notification',
        lang: 'zh_CN'
    };
    var EVENTS = ['click', "show", "error", "close"];
    var LEGACY_EVENTS = ["display", "close"];

    function clone(o) {
        return JSON.parse(JSON.stringify(o));
    }

    function extend(p, c) {
        var result = clone(p);

        for(var i in c) {
            result[i] = c[i];
        }

        return result;
    }

    var isAllowed = function(cb) {
        if(isLegacy) {
            isAllowed = function (cb) {
                cb(Notify.checkPermission() === 0);
            };
        } else {
            isAllowed = function(cb) {
                Notify.requestPermission(function(choice) {
                    cb(choice === GRUNTED);
                });
            };
        }

        isAllowed(cb);
    };

    var bindEvents = function (notification, options) {

        if (isLegacy) {
            bindEvents = function(notification, options) {
                var keys = Object.keys(options || {});

                LEGACY_EVENTS.forEach(function(evt){
                    if(keys.indexOf(evt) > -1) {
                        notification['on' + evt] = options[evt];
                    }
                });
            };
        } else {
            bindEvents = function(notification, options) {
                var keys = Object.keys(options || {});

                EVENTS.forEach(function(evt){
                    if(keys.indexOf(evt) > -1) {
                        notification['on' + evt] = options[evt];
                    }
                });
            };
        }

        bindEvents(notification, options);
    };

    var sendNotification = function  (title, options) {
      // Memoize based on feature detection.
      if (isLegacy) {
        sendNotification = function(title, options) {
            options = extend(DEFAULT_SETTING, options);

            var n = Notify.createNotification(options.icon, title, options.body);
            bindEvents(n, options);

            n.show();

            return n;
        };
      } else {
        sendNotification = function (title, options) {
            options = extend(DEFAULT_SETTING, options);

            var n = new Notify(title, options);
            bindEvents(n, options);

            return n;
        };
      }

      sendNotification(title, options);
    };

    root.gNotify = {
        isAllowed: isAllowed,
        sendNotification: sendNotification
    };

})(window);
