/* global Runtime, chrome, Config, Logger, WebCall, Utils, Tracker */

/*
 * Chrome runtime helper:
 - commnucate with web page and other extension or app
 - manifest access
 */
(function(root, $) {
    'use strict';

    var appDetails = chrome.app.getDetails();
    var handlers = {};
    var extHandlers = {};
    var Cookie = root.Cookie;
    var Tab = root.Tab;
    var gStorage = root.gStorage;

    var buffer;
    var blob

    root.BLOCK_ALL_WEBREQUEST = false;
    root.BLOCK_SITE_WEBREQUEST = false;

    function lagouFormatToFile(buffer) {
        var mimeType = '';
        var fileExt = '';

        try {
            var int32View = new Int32Array(buffer);
            console.log('int32View[0]: ' + int32View[0]);
            switch(int32View[0]) {
                case -535703600:
                    mimeType = "application/msword";
                    fileExt = '.doc';
                    break;
                default:
                    mimeType = 'application/pdf';
                    fileExt = '.pdf';
                    break;
            }
        } catch (e) {
            mimeType = 'application/pdf';
            fileExt = '.pdf';
        }

        var file = new File([blob], '拉勾简历' + fileExt, {
            type: mimeType
        });

        var formData = new FormData();
        formData.append('file', file);
        formData.append('data', JSON.stringify({
            type: 'candidate'
        }));

        return formData;
    }

    // message from content script
    chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
        if (sender.tab) {
            // Logger.log("message from content script: ", arguments);
            var h = handlers[request.type];

            if (h) {
                h.forEach(function(handler) {
                    handler(request, sender, sendResponse);
                });
            }

            // make 'sendResponse' support async
            return true;
        }
    });

    // message from web page or chrome app
    chrome.runtime.onMessageExternal.addListener(function(request, sender, sendResponse) {
        var srouce = sender.tab ? "web page" : "chrome-app";

        // Logger.log("Message from " + srouce + ": ", arguments);
        var h = extHandlers[request.type];

        if (h) {
            h.forEach(function(handler) {
                handler(request, sender, sendResponse);
            });
        }
    });

    root.Runtime = {
        version: appDetails.version,

        // mock the request from web page
        mockExtenalSendMsg: function(data, tabId) {
            var h = extHandlers[data.type];

            if (h) {
                h.forEach(function(handler) {
                    handler(data, {
                        tab: {
                            id: tabId
                        }
                    });
                });
            }
        },

        getExtensionInfo: function() {
            return appDetails;
        },

        onExtMessage: function(evtType, evtHandler) {
            var types = evtType.split(',');

            types.forEach(function(type) {
                var h = extHandlers[type] || (extHandlers[type] = []);

                h.push(evtHandler);
            });
        },

        onMessage: function(evtType, evtHandler) {
            var types = evtType.split(',');

            types.forEach(function(type) {
                var h = handlers[type] || (handlers[type] = []);

                h.push(evtHandler);
            });
        }
    };

    // subscribe to extension host name changed
    Runtime.onExtMessage("request-extension-hostname", function (request, sender) {
        gStorage.getHost().then(function (host) {
            Tab.sendMsgToTab(sender.tab.id, {
                type: 'get-extension-hostname',
                value: host
            });
        });
    });

    // subscribe to webcall native client version
    Runtime.onExtMessage("request-native-version", function (request, sender) {
        Tab.sendMsgToTab(sender.tab.id, {
            type: 'get-native-version',
            value: root.WebCall.nativeVersion
        });
    });

    Runtime.onExtMessage('request-gllue-token', function (request, sender) {
        root.WebCall.createGllueToken(request.value, function (token) {
            Tab.sendMsgToTab(sender.tab.id, {
                type: 'get-gllue-token',
                value: token
            });
        });
    });

    // subscribe to open extension page
    Runtime.onExtMessage("open", function getHostname(request, sender) {
        if (request.page) {
            Tab.showPageByFileName(request.page + '.html');
        }
    });

    // subscribe to Get/Post JSON from content script
    Runtime.onMessage("ajax", function onAjax(request, sender, sendResponse) {
        gStorage.getHost().then(function (host) {
            switch(request.action) {
                case 'get':

                    $.getJSON(host + request.url).done(sendResponse);
                    break;

                case 'post':

                    $.post(host + request.url, request.data).done(sendResponse);
                    break;
            }
        });
    });

    Runtime.onMessage("lagouCheckDupe", function onCheckDupeExtactly(request, sender, sendResponse) {
        var data = {};
        var xhr = new XMLHttpRequest();
        var url = request.url;
        data.html = buffer;
        var formData =lagouFormatToFile();
        xhr.open('POST', url, true);
        xhr.send(formData);
        xhr.onload = function (e) {
            var res;
            if (this.readyState === 4 && this.status === 200) {
                res = JSON.parse(this.responseText);
            } else {
                if (this.status === 404) {
                    console.error(this.statusText);
                } else {
                    try {
                        res = JSON.parse(this.responseText);
                        console.error(JSON.parse(this.responseText).message);
                    } catch (err) {
                        // Utils.alert('error', this.statusText);
                        console.error(this.statusText);
                    }
                }

            }

            sendResponse(res);
        }
    });


    // subscribe to upload lagou resume
    Runtime.onMessage("uploadLagouResume", function onUploadLagouResume(request, sender, sendResponse) {
        var oReq = new XMLHttpRequest();
        oReq.open("GET", 'https://easy.lagou.com/resume/download.htm?resumeId=' + request.resumeId, true);
        oReq.responseType = "blob";

        oReq.onload = function (oEvent) {
            blob = oReq.response; // Note: not oReq.responseText
            var reader = new FileReader();

            reader.readAsArrayBuffer(blob);

            reader.onload = function (event) {
                buffer = reader.result;
                var formData =lagouFormatToFile();

                gStorage.getHost().then(function (host) {
                    var requestURL = host + '/rest/file/upload';
                    var xhr = new XMLHttpRequest();

                    // on load
                    xhr.onload = function(e) {
                        var res;
                        // on success
                        if (this.readyState === 4 && this.status === 200) {
                            res = JSON.parse(this.responseText);
                        } else {
                            if (this.status === 404) {
                                console.error(this.statusText);
                            } else {
                                try {
                                    res = JSON.parse(this.responseText);
                                    console.error(JSON.parse(this.responseText).message);
                                } catch (err) {
                                    // Utils.alert('error', this.statusText);
                                    console.error(this.statusText);
                                }
                            }

                        }

                        sendResponse(res);
                    };

                    xhr.open('POST', requestURL, true);
                    xhr.send(formData);

                });
            };
        };

        oReq.send();
    });


    // error handling from content script
    Runtime.onMessage("errorHandling", function onErrorHandling(request, sender, sendResponse) {
        var data = request.data;
        Logger.store(data.msg, data.stack, {
            jsfile: data.url,
            line: data.line
        }, data.page_url);
    });


    // message from content script to get user context
    Runtime.onMessage('user/context', function(request, sender, sendResponse) {

        Utils.getUserContext(sendResponse);

    });

    // message from content script site position publish page to get cookie
    Runtime.onMessage('site/cookie', function(request, sender, sendResponse) {
        Cookie.getCookieByURL(request.data.url, sendResponse);

        Logger.log('Received request for ' + request.data.site + ' cookie!');
    });

    // message from content script site position publish page to get cookie
    Runtime.onMessage('phonebox/check', function(request, sender, sendResponse) {
        sendResponse(root.WebCall.clientInstalled);
    });

    // message from content script to block all webrequest.
    Runtime.onMessage('block/webrequest', function(request, sender, sendResponse) {
        if (request.data && request.data.all) {
            root.BLOCK_ALL_WEBREQUEST = !root.BLOCK_ALL_WEBREQUEST;
            if (root.BLOCK_ALL_WEBREQUEST) {
                Logger.log('Received request for block all jobboard webrequest!!!');
            } else {
                Logger.log('Received request for UNBLOCK all jobboard webrequest.');
            }
        }
        if (request.data && request.data.site) {
            root.BLOCK_SITE_WEBREQUEST = !root.BLOCK_SITE_WEBREQUEST;
            if (root.BLOCK_SITE_WEBREQUEST) {
                Logger.log('Received request for block jobboard post job webrequest!!!');
            } else {
                Logger.log('Received request for UNBLOCK jobboard post job webrequest.');
            }

        }
        sendResponse({
            'BLOCK_SITE_WEBREQUEST': root.BLOCK_SITE_WEBREQUEST,
            'BLOCK_ALL_WEBREQUEST': root.BLOCK_ALL_WEBREQUEST
        });
    });

}(window, jQuery));
