/* global gStorage, Tab, chrome, Config */

/*
 * Util functions for CRUD chrome.storage
 */
(function(root) {
    'use strict';

    var handlers = [];
    var hostHandlers = [];
    var HOST = '';
    var HOST_NAME = Config.HOST_NAME;

    // Everytime when 'HOST' updated.
    chrome.storage.onChanged.addListener(function(changes, namespace) {
        var hostChange = changes[HOST_NAME];

        if (hostChange && hostChange.newValue !== hostChange.oldValue) {
            HOST = hostChange.newValue;

            hostHandlers.forEach(function (handler) {
                handler(HOST);
            });
        }

        handlers.forEach(function(handler) {
            handler(changes, namespace);
        });

    });

    function onChanged(cb) {
        handlers.push(cb);
    }

    function onHostChange(cb) {
        hostHandlers.push(cb);
    }

    function set(obj) {
        return new Promise(function(resolve, reject){
            chrome.storage.sync.set(obj, resolve);
        });
    }

    function get(key) {
        return new Promise(function(resolve, reject){
            chrome.storage.sync.get(key, resolve);
        });
    }
    function getHost() {

        if (!HOST) {
            return get(HOST_NAME).then(function(items){
                HOST = items[HOST_NAME];

                // If HOST is not configed will show option page automatically
                if (!HOST) {
                    if (root.Tab) {
                        Tab.showPageByFileName('options.html');
                    }
                }
                return HOST;
            });
        } else {
            return Promise.resolve(HOST);
        }
    }

    function setHost(val) {
        var data = {};
        data[HOST_NAME] = val;
        return set(data).then(function () {
            Config.HOST = HOST = val;
        });
    }

    root.gStorage = {
        set: set,
        get: get,
        onHostChange: onHostChange,
        onChanged: onChanged,
        getHost: getHost,
        setHost: setHost
    };

})(window);
