/* global WebRequest, Config, Logger, chrome, Utils, Runtime, saas, gStorage, Tracker */

(function(root){
	"use strict";

    // request and reqsponse event handlers
	var headerHandlers = [];
    var beforeRequestHandlers = [];
    var responseStartedHandlers = [];

    // setting options
    var opt_extraInfoSpecReq = ["blocking", "requestHeaders"];
    var opt_extraInfoSpecReqBody = ["blocking", "requestBody"];
    var opt_extraInfoSpecRes = ["responseHeaders"];

	function walker(o, handler) {
		for(var i in o) {
			if(o.hasOwnProperty(i)) {
				handler(o[i], i);
			}
		}
	}

	function arrayToObj(a) {
		var result = {};

		for(var i=0, len= a.length; i<len; i++) {
			result[a[i].name] = a[i].value;
		}

		return result;
	}

	function objToArr(o) {
		var result = [];

		walker(o, function(value, name) {
			result.push({
				name: name,
				value: value
			});
		});

		return result;
	}

	function clearOnceHandler(handlers) {
		return handlers.filter(function(handler){
            return !handler.once;
        });
	}

    function onBeforeSendHeadersListener(details) {
        var headers;
        headers = arrayToObj(details.requestHeaders);

        headers = headerHandlers.reduce(function(curr, handler) {
            try{
                curr = handler(details, curr);
            } catch (e){
                console.error(e);
                Logger.log(e);
            }
            return curr;
        }, headers);

        headerHandlers = clearOnceHandler(headerHandlers);
        return {requestHeaders: objToArr(headers)};
    }

    function onBeforeRequestLinstener(details) {
        var retData;
        beforeRequestHandlers.forEach( function (handler) {
            try{
                var ret = handler(details);
                retData = ret || retData;
            } catch (e){
                console.error(e);
                Logger.log(e);
            }
        });

        beforeRequestHandlers = clearOnceHandler(beforeRequestHandlers);

        if (root.BLOCK_ALL_WEBREQUEST) {
            var exp = new RegExp(root.saas.Config.siteUrlReg);
            if (exp.test(details.url)) {
                Logger.warn("URL BLOCKED: "+details.url);
                return {cancel: true};
            }
        }
        return retData;
    }

    function onResponseStartedListener(details) {
        responseStartedHandlers.forEach( function (handler) {
            try{
                handler(details);
            } catch (e){
                console.error(e);
                Logger.log(e);
            }
        });

        responseStartedHandlers = clearOnceHandler(responseStartedHandlers);
    }

	root.WebRequest = {
        start: function(config) {
            console.log('WebRequest start...');
            // default filter:
            // 默认即使插件无法连接 saas， 也需要过滤51和智联的请求，
            // 因为glluechannel在某些refresh请求需要更改 referrer.
            var filter = {
                urls: [
                    "*://*.liepin.com/*",
                    "*://*.zhaopin.com/*",
                    "*://*.lagou.com/*",
                    "*://*.51job.com/*",
                    "*://*.58.com/*",
                    "*://*.yingjiesheng.com/*",
                    "*://*.linkedin.com/*",
                    "*://*/*extension/message*"
                ]
            };
            if (config) {
                filter = {
                    urls: config.webrequestUrls
                };
            }

            chrome.webRequest.onBeforeSendHeaders.removeListener(onBeforeSendHeadersListener);
            chrome.webRequest.onBeforeRequest.removeListener(onBeforeRequestLinstener);
            chrome.webRequest.onResponseStarted.removeListener(onResponseStartedListener);

            // intercept request and check header
            chrome.webRequest.onBeforeSendHeaders.addListener(
                onBeforeSendHeadersListener,
                filter,
                opt_extraInfoSpecReq
            );

            // intercept request and check request body
            chrome.webRequest.onBeforeRequest.addListener(
                onBeforeRequestLinstener,
                filter,
                opt_extraInfoSpecReqBody);

            // intercept response and check response header
            chrome.webRequest.onResponseStarted.addListener(
                onResponseStartedListener,
                filter,
                opt_extraInfoSpecRes);
        },
        // before send request body
        addBeforeRequestHandler: function(func){
            beforeRequestHandlers.push(func);
        },
        addBeforeRequestHandlerOnce:function(func){
            func.once = true;
            beforeRequestHandlers.push(func);
        },
        // before send headers
        addHeaderHandler: function(func){
            headerHandlers.push(func);
        },
        addHeaderHandlerOnce: function(func){
            func.once = true;
            headerHandlers.push(func);
        },

        // before response
        addReponseHandler: function(func){
            responseStartedHandlers.push(func);
        },
        addReponseHandlerOnce: function(func){
            func.once = true;
            responseStartedHandlers.push(func);
        }

	};

    root.ServerStub.startListenHost();
    root.WebRequest.start(saas.Config);

    gStorage.onHostChange(function () {
        root.ServerStub.stopListenHost();
        root.ServerStub.startListenHost();
    });
})(window);
