/* global chrome, WebCall, Runtime, Logger, Tab, Cookie, Utils, Config, Tracker */

(function (root) {
    "use strict";


    root.WebCall = root.WebCall || {};

    root.WebCall.webcallSessionMap = {};
    root.WebCall.nativeEventMapping = {};

    function sendMsgToNative(message, callback) {
        var uuid = Utils.UUID();
        message['uuid'] = uuid;

        if (callback) {
            root.WebCall.nativeEventMapping[uuid] = callback;
        }

        try {
            var context = $.extend(message, {
                nativeVersion: root.WebCall.nativeVersion || '',
                session_uuid: Session.currentSession.uuid
            });
        } catch (e) {

        }
        root.WebCall.sendMsgToNative(message);
    }

    root.WebCall.getNativeVersion = function () {
        var message = {
            'type': 'api',
            'data': {
                'func': 'version',
                'args': []
            }
        };
        sendMsgToNative(message, function (res) {
            root.WebCall.nativeVersion = res.version;
        });
    };

    root.WebCall.createGllueToken = function (key, callback) {
        var message = {
            'type': 'api',
            'data': {
                'func': 'create_gllue_token',
                'args': [key]
            }
        };
        sendMsgToNative(message, function (res) {
            if (callback) {
                callback(res);
            }
        });
    };

    root.WebCall.checkNativeStatus = function (callback) {
        var message = {
            'type': 'api',
            'data': {
                'func': 'check_box_status',
                'args': []
            }
        };
        sendMsgToNative(message, function (res) {
            if (callback) {
                callback(res);
            }
        });
    };


    function formatDatetime(time) {
        var date = new Date(time);
        return date.getFullYear() + '-' + (date.getMonth() + 1) +  '-' + date.getDate() + ' ' + date.getHours() + ':' +  date.getMinutes() + ':' + date.getSeconds();
    }

    var Session = function () {
        // status: ['dial_out', 'call_in', 'normal']
        this.status = undefined;
        this.candidateId = undefined;
        this.mobile = undefined;
        this.recordFile = undefined;
        this.start = undefined;
        this.end = undefined;
        this.duration = undefined;
        this.miss = undefined;
        this.uuid = Utils.UUID();

        this.setStatus = function (status) {
            this.status = status;
        };

        this.setStartTime = function () {
            this.start = Date.now();
        };

        this.setEndTime = function () {
            if (!this.end) {
                this.end = Date.now();
            }
        };

        this.startRecord = function () {
            var me = this;
            var message = {
                'type': 'api',
                'data': {
                    'func': 'start_record',
                    'args': [me.mobile]
                }
            };

            sendMsgToNative(message,  function (res) {
                me.recordFile = res.data;
            });
        };

        this.endRecord = function () {
            var message = {
                'type': 'api',
                'data': {
                    'func': 'stop_record'
                }
            };

            sendMsgToNative(message);
        };

        this.uploadRecord = function (callback) {
            var me = this;
            Cookie.getCookieByHost(Utils.getHostNameByURL(Config.HOST)).then(function (cookie){
                var message = {
                    'type': 'api',
                    'data': {
                        'func': 'upload_file',
                        'args': [me.recordFile]
                    },
                    extra: {
                        'record_file_name': me.recordFile,
                        'session_uuid': me.uuid,
                        'url': Config.HOST + '/rest/phonebox/add',
                        'cookie': {'uid': cookie.uid}
                    }
                };

                sendMsgToNative(message, callback);
            });

        };

        this.sendRecord = function () {
            var me = this;

            if (!me.candidateId || !me.start || !me.end) {
                return;
            }

            $.ajax({
                url:  Config.HOST + '/rest/phonebox/add',
                type: 'post',
                data: {
                    data: JSON.stringify({
                        'candidate_id': me.candidateId,
                        'mobile': me.mobile,
                        'start': formatDatetime(me.start),
                        'end': formatDatetime(me.end),
                        'duration': (me.end - me.start) / 1000,
                        'type': me.status,
                        'session_uuid': me.uuid
                    })
                }
            });
        };

        this.dialOut = function () {
            var me = this;
            var message = {
                'type': 'api',
                'data': {
                    'func': 'call',
                    'args': [me.mobile]
                }
            };

            sendMsgToNative(message);
        };

        this.stopDialOut = function () {

        };

        this.reconnect = function reconnect() {
            var me = this;
            var message = {
                'type': 'api',
                'data': {
                    'func': 'reconnect'
                },
                extra: {
                    'session_uuid': me.uuid
                }
            };

            sendMsgToNative(message);
        };

        this.sessionEnd = function () {
            var me = this;
            this.endRecord();
            this.setEndTime();
            this.sendRecord();
            this.reconnect();

            Session.lastSession = Session.currentSession;
        };


        Session.currentSession = this;
    };

    // keep current session in static variable
    Session.currentSession = undefined;

    Session.lastSession = undefined;

    root.WebCall.Session = Session;
}(window));
