/* global chrome, WebCall, Runtime, gStorage, Logger, Tab, Cookie, Utils, Config, Tracker */

/* @flow */

/*
 * Web call module is used to serve calling from web page.
 * */

(function(root) {
    "use strict";

    root.WebCall = root.WebCall || {};

    var PREFIX = "webcall";
    var TAB_ID;

    var nativeTimer;
    var nativeHandlers = {};
    var nativeDisconnectHandlers = {};
    var port;
    var Session = root.WebCall.Session;
    var getVersionMaxTry = 0;
    var outSourceEventHandlers = {};

    var EVENT_NAME_MAPPING = {
        1: 'onPhoneHook',
        2: 'onPhoneHang',
        6: 'onDialEnd',
        3: 'onCallIn',
        4: 'onGetCallID',
        5: 'onStopCallIn',
        15: 'onGetDTMFChar',
        16: 'onRemoteHook',
        17: 'onRemoteHang',
        18: 'onBusy',
        19: 'onDialTone',
        20: 'onPhoneDial',
        21: 'onRingBack',
        199: 'onDevErr',
        25: 'onRefuseEnd',
        27: 'onPSTNFree',
        106: 'onDoStartDial'
    };

    var EventHandlers = function () {
        function onPhoneHook(data) {
            if (Session.currentSession) {
                Session.currentSession.setStatus(Session.currentSession.status);

                if (Session.currentSession.status === 'call_in') {
                    Session.currentSession.setStartTime();
                    Session.currentSession.startRecord();
                }

            } else {
                //没有通话事件
                new Session();
            }
        }

        function onPhoneHang(data) {
            if (Session.currentSession) {
                Session.currentSession.sessionEnd();
            }
        }

        function onDialEnd(data) {

        }

        function onCallIn(data) {
            new Session();
            Session.currentSession.setStatus('call_in');
        }

        function onGetCallID(data) {
            if (WebCall.popupId) {
                closeWebcallPopup(WebCall.popupId);
            }

            if (!Session.currentSession) {
                new Session();
            }
            Session.currentSession.mobile = data['szData'];

            var params = window.encodeURI($.param({
                action: 'call_in',
                session_id: root.Utils.UUID(),
                tel: Session.currentSession.mobile,
                check_mobile: Session.currentSession.mobile
            }));

            openWebcallPopup(params);
        }

        function onStopCallIn(data) {
            // reset driver
            if (Session.currentSession) {
                Session.currentSession.reconnect();
            }
        }

        function onGetDTMFChar(data) {

        }

        function onRemoteHook(data) {
            if (!Session.currentSession) {
                new Session();
            }

            Session.currentSession.setStartTime();
            Session.currentSession.startRecord();
        }

        function onRemoteHang(data) {
            if (Session.currentSession) {
                if (Session.currentSession.status === 'call_in') {

                } else if (Session.currentSession.status === 'dial_out') {

                }

                Session.currentSession.sessionEnd();
            }
        }

        function onBusy(data) {
        }

        function onDialTone(data) {
        }

        function onPhoneDial(data) {
            if (!Session.currentSession) {
                new Session();
            }
            Session.currentSession.mobile = data['szData'];
            Session.currentSession.setStatus('dial_out');
        }

        function onRingBack(data) {
        }

        function onDevErr(data) {
        }

        function onRefuseEnd(data) {
            // reset driver
            if (Session.currentSession) {
                Session.currentSession.reconnect();
            }
        }

        function onPSTNFree(data) {
        }

        function onDoStartDial(data) {
            if (!Session.currentSession) {
                new Session();
            }

            Session.currentSession.setStatus('dial_out');
            Session.currentSession.mobile = data['szData'];
        }

        var EVENT_MAPPING = {
            //本地电话机摘机事件
            1: onPhoneHook,
            //本地电话机挂机事件
            2: onPhoneHang,
            //调用开始拨号后，全部号码拨号结束
            6: onDialEnd,
            //外线通道来电响铃事件
            //BRI_EVENT.lResult		为响铃次数
            //BRI_EVENT.szData[0]=&#34;0&#34;	开始1秒响铃
            //BRI_EVENT.szData[0]=&#34;1&#34;	为1秒响铃完成，开始4秒静音
            3: onCallIn,
            //获取本次呼入的来电号码
            4: onGetCallID,
            //对方停止呼叫(产生一个未接电话)
            5: onStopCallIn,
            //线路接通时收到DTMF码事件
            //该事件不能区分通话中是本地话机按键还是对方话机按键触发
            15: onGetDTMFChar,
            //拨号后,被叫方摘机事件
            //原因：
            //该事件只适用于拨打是标准信号音的号码时，也就是拨号后带有标准回铃音的号码。
            //如：当拨打的对方号码是彩铃(彩铃手机号)或系统提示音(179xx)都不是标准回铃音时该事件无效。
            16: onRemoteHook,
            //挂机事件
            //如果线路检测到被叫方摘机后，被叫方挂机时会触发该事件，不然被叫方挂机后就触发onBusy事件
            //该事件或者onBusy的触发都表示PSTN线路已经被断开
            17: onRemoteHang,
            //检测到忙音事件,表示PSTN线路已经被断开
            18: onBusy,
            //本地摘机后检测到拨号音
            19: onDialTone,
            //只有在本地话机摘机，没有调用软摘机时，检测到DTMF拨号
            20: onPhoneDial,
            //电话机拨号结束呼出事件。
            //也就时电话机拨号后接收到标准回铃音或者15秒超时
            //BRI_EVENT.lResult=0 检测到回铃音# 注意：如果线路是彩铃是不会触发该类型
            //BRI_EVENT.lResult=1 拨号超时
            //BRI_EVENT.lResult=2 动态检测拨号码结束(根据中国大陆的号码规则进行智能分析，仅做参考)
            //BRI_EVENT.szData[0]=&#34;1&#34; 软摘机拨号结束后回铃了
            //BRI_EVENT.szData[0]=&#34;0&#34; 电话机拨号中回铃了
            21: onRingBack,
            //设备错误
            199: onDevErr,
            //拒接完成
            25: onRefuseEnd,
            //PSTN线路断开,线路进入空闲状态
            //当前没有软摘机而且话机也没摘机
            27: onPSTNFree,
            //开始软件拨号
            //BRI_EVENT.szData	准备拨的号码
            106: onDoStartDial
        };

        function invoke(data) {
            if (!data) {
                return;
            }
            var eventId = data.message_id;
            if (EVENT_MAPPING[eventId]) {
                var func = EVENT_MAPPING[eventId];
                console.log(EVENT_NAME_MAPPING[eventId]);
                func(data);
                var handlers = outSourceEventHandlers[EVENT_NAME_MAPPING[eventId]];
                if (handlers && handlers.length) {
                    for (var i = 0, l = handlers.length; i < l; i++) {
                        if (typeof handlers[i] === 'function') {
                            handlers[i](data);
                        }
                    }
                }
            }
        }

        return {
            invoke: invoke
        };
    };

    var eventHandlers = new EventHandlers();

    function errorHandler() {
        if(chrome.runtime.lastError && chrome.runtime.lastError.message) {
            console.warn(" [Chrome Runtime Exception] ", chrome.runtime.lastError.message);
        }
    }

    /*
     *  sometimes, the native client closed somehow
     *  the timer is responsible to try connect over and over
     *  */
    function nativeConnectTimerHandler(action) {
        if(action === 'open' && !port) {

            nativeTimer = window.setTimeout(function() {
                tryConnectNativeBox();
            }, 10000);

        } else if(action === 'close') {

            window.clearTimeout(nativeTimer);
            nativeTimer = null;

        }
    }

    function notifyPhoneBoxInstalledStatus(isInstalled) {
        // notify webcall box not to init
        if(WebCall && WebCall.Hub) {
            WebCall.Hub.checkedBoxInstallStatus.dispatch(isInstalled);
        }
    }

    function onConnectNative(request, sender, sendResponse) {
        /*
         request: {type: 'event': ret:{}}
                {type: 'api', uuid: 'string', ret: {}}
         */

        if (request.type === 'event') {
            eventHandlers.invoke(request.ret);
        } else if (request.type === 'api') {
            var handler = root.WebCall.nativeEventMapping[request.uuid];
            if (typeof handler === 'function') {
                handler(request.ret);
                delete root.WebCall.nativeEventMapping[request.uuid];
            }
        }

    }

    function onDisConnectNative(request, sender, sendResponse) {
        // Logger.log("Message from Native Client: ", request);

        port = null;

        nativeConnectTimerHandler('open');

        notifyPhoneBoxInstalledStatus(false);

        var h =  nativeHandlers[request.event];

        if(h) {
            h.forEach(function(handler) {
                handler(request, sender, sendResponse);
            });
        }
    }

    function tryConnectNativeBox() {
        try{
            port = chrome.runtime.connectNative(Config.NATIVE_CLIENT_HOST);
        } catch(e) {
            Logger.error(e);
        }

        if (port) {

            port.onMessage.removeListener(onConnectNative);
            port.onDisconnect.removeListener(onDisConnectNative);

            port.onMessage.addListener(onConnectNative);
            port.onDisconnect.addListener(onDisConnectNative);

            if (!root.WebCall.nativeVersion && getVersionMaxTry <= 3) {
                getVersionMaxTry++;
                root.WebCall.getNativeVersion();

                var currentCheck = 0;
                var checkTimeId = window.setInterval(function () {
                    currentCheck++;
                    root.WebCall.checkNativeStatus(function (res) {
                        root.WebCall.clientInstalled = res;
                    });
                    if (currentCheck >= 10) {
                        window.clearInterval(checkTimeId);
                    }
                }, 1000);

            }

        } else {

        }

    }

    function closeWebcallPopup(windowId) {
        chrome.windows.remove(windowId);
    }

    function openWebcallPopup(params) {
        var screenHeight = root.screen.availHeight;
        var screenWidth = root.screen.availWidth;
        var width = 330;
        var height = 326;
        var left = ( screenWidth - width ) / 2;
        var top = ( screenHeight - height ) / 2;

        chrome.windows.create({
            url:  'webcall-popup.html' + "?" + params,
            type: "popup",
            width: Math.round(width),
            height: Math.round(height),
            left: Math.round(left),
            top: Math.round(top)
        });
    }

    var App = $.extend(root.WebCall,{
        registerEventHandler: function (funcName, func) {
            if (!outSourceEventHandlers[funcName]) {
                outSourceEventHandlers[funcName] = [];
            }
            outSourceEventHandlers[funcName].push(func);
        },
        onNativeMessage: function(evtType, evtHandler) {
            var types = evtType.split(',');

            types.forEach(function(type) {
                var h = nativeHandlers[type] || (nativeHandlers[type] = []);

                h.push(evtHandler);
            });
        },

        onNativeDisConnect: function(evtType, evtHandler) {
            var types = evtType.split(',');

            types.forEach(function(type) {
                var h = nativeDisconnectHandlers[type] || (nativeDisconnectHandlers[type] = []);

                h.push(evtHandler);
            });
        },

        sendMsgToNative: function(message) {
            if(!port) {
                return;
            }

            port.postMessage(message);

            errorHandler();
        },
        initEvents: function() {
            function beforeActionHandler(request) {
                Logger.log('[BOX MSG] ', request.event, request.data);

                App.status(request.type);
            }

            Runtime.onExtMessage(PREFIX, function(request, sender) {
                if (WebCall.popupId) {
                    closeWebcallPopup(WebCall.popupId);
                }

                TAB_ID = sender.tab.id;

                request.value.tab_id = TAB_ID;

                /*
                 * request : {
                 *   type: "webcall",
                 *   value: {
                 *       mobile: 12323,
                 *       otherdata ....
                 *   }
                 * }
                 * */

                var params = window.encodeURI($.param({
                    action: 'dial_out',
                    session_id: root.Utils.UUID(),
                    tel: request.value.dialMobile,
                    tab_id: TAB_ID,
                    check_mobile: request.value.mobile
                }));

                openWebcallPopup(params);
            });
        },

        status: function(status) {
            if(status) {
                App._status = status;
            }

            return App._status;
        }
    });

    tryConnectNativeBox();

    App.initEvents();

})(window);
