/* global $, chrome, Config, Mustache, Utils, pako, Logger */
(function (root) {
    var URL = {
        'detail': '/crm#candidate/detail?id={0}',
        'autoshare': '/rest/candidate/autoshare',
        'uploadresume': "/rest/file/htmlfile"
    };

    var Utils = root.Utils;

    var APP = {
        init: function (config) {
            var me = this;
            var defaultConfig = {
                changeDomin : null,
                notSupportDupe : false,
                changexHtml : null
            };
            $.extend(me, defaultConfig, config);
        },
        //获取域名
        getDomain: function(){
            var me = this;
            if (window.location.protocol.indexOf('chrome') === 0){
                return null;
            }
            var hostname = window.location.hostname;
            var domain =  hostname.replace(/^(.*?)\.(.*?)\..*/g, "$2");
            if (me.changeDomin) {
                domain = me.changeDomin(domain);
            }
            return domain;
        },
        invertCandidate: function (res) {
            var data = res.data.candidate.length ? res.data.candidate[0]: null;
            return data;
        },
        invertSimilar: function (res) {
            var data = res.data || [];
            if (res.candidate) {
                var exceptId = res.candidate.id;
                data = data.filter(function (item) {
                    return item.id !== exceptId;
                });
                data = data.length ? data: [];
            }
            return data;
        },
        invertImportBack: function (res) {
            return res;
        },
        //精确查重
        checkDupeExtactly: function () {
            var me = this;
            me.host = arguments[0];
            var deferred = $.Deferred();
            var p = $.Deferred().resolve({});
            var URL = {
                'extactly': '/rest/candidate/check_exists/'
            };
            var url = URL['extactly'];
            if (me.notSupportDupe) {
                deferred.resolve({
                    status: false
                });
                return p;
            }
            p.then(function(data){
                me.getHTML(me.deflate).then(function (res) {
                    data.html = res;
                    Utils.postJSON(url, {
                        data: JSON.stringify(data)
                    }).then(function (dupRes) {
                        //如果候选人没有权限自动共享
                        if (dupRes.data && dupRes.data.candidate && dupRes.data.candidate.length &&  dupRes.data.candidate[0].islimited) {
                            var dupInfo = dupRes.data.dup_info[0];
                            var requestData = {};
                            requestData[dupInfo.type.toLowerCase()] = dupInfo.value;
                            Utils.postJSON(URL.autoshare, {
                                data: JSON.stringify(requestData)
                            }).then(function (res) {
                                if (res.status) {
                                    Utils.getJSON('/rest/candidate/' + res.data).then(function (res) {
                                        dupRes.data.candidate[0] = res;
                                        deferred.resolve(dupRes);
                                    });
                                } else {
                                    deferred.resolve(dupRes);
                                }

                            });
                        } else {
                            deferred.resolve(dupRes);
                        }

                    }, deferred.reject);
                });
            });

            return deferred;
        },
        //模糊查重
        checkDupeFuzzy: function () {
            var me = this;
            var deferred = $.Deferred();
            var p = $.Deferred().resolve({});
            var URL = {
                'fuzzy': '/rest/candidate/check_html_content'
            };
            var url = URL['fuzzy'];
            if (me.notSupportDupe) {
                deferred.resolve({
                    status: false
                });
                return p;
            }
            p.then(function(data){
                me.getHTML(me.deflate).then(function (res) {
                    data.html = res;
                    Utils.postJSON(url, {
                        data: JSON.stringify(data)
                    }).then(function (dupRes) {
                        //如果候选人没有权限自动共享
                        if (dupRes.data && dupRes.data.candidate && dupRes.data.candidate.length &&  dupRes.data.candidate[0].islimited) {
                            var dupInfo = dupRes.data.dup_info[0];
                            var requestData = {};
                            requestData[dupInfo.type.toLowerCase()] = dupInfo.value;
                            Utils.postJSON(URL.autoshare, {
                                data: JSON.stringify(requestData)
                            }).then(function (res) {
                                if (res.status) {
                                    Utils.getJSON('/rest/candidate/' + res.data).then(function (res) {
                                        dupRes.data.candidate[0] = res;
                                        deferred.resolve(dupRes);
                                    });
                                } else {
                                    deferred.resolve(dupRes);
                                }

                            });
                        } else {
                            deferred.resolve(dupRes);
                        }

                    }, deferred.reject);
                });
            });

            return deferred;
        },

        getRawHTML: function () {
            var me = this;
            var $html = $('html').clone();
            var html;

            $html.find('.gllue-toolbar').remove();

            if (me.changeHtml) {
                $html = me.changeHtml($html);
            }
            return html = $html[0].outerHTML;
        },

        //获取HTML
        getHTML: function(deflate) {
            var me = this;
            var html = this.getRawHTML();
            if (deflate) {
                return $.Deferred().resolve(deflate(html));
            }
            return $.Deferred().resolve(html);
        },
        deflate: function(rawString) {
            return window.btoa(pako.deflate(rawString, { to: 'string' }));
        },

        uploadCallback: function (res){
            return $.Deferred().resolve();
        },
        mergeCandidateCallback: function (res){
            var me = this;
            if (res.status) {
                var uuidname = res.data.split('?file=')[1];
                var url = me.host + '/data-merge#?uuidname=' + uuidname + '&ids=' + me.id + '&model=candidate&suffix=candidate';
                window.open(url);
            } else {
                console.error(res.message);
            }
        },
        importCallback: function (res) {
            var me = this;
            return Utils.openAddCandidate(res);
        },
        importOneClickDataCallBack: function () {
            return $.Deferred().resolve();
        },
        upload: function (extractData) {
            var me = this;
            me.host = extractData.host;
            me.joborderId = extractData.joborderId;
            return me.extracting(extractData).then(function (res) {
                // lagou的me.extracting会返回res=null,会对这个传参数有影响吗?
                return me.uploadResume(me.uploadCallback)(res);
            });
        },
        mergeCandidate: function (extractData) {
            var me = this;
            me.host = extractData.host;
            me.id = extractData.id;
            me.joborderId = extractData.joborderId;
            return me.extracting(extractData).then(function (res) {
                return me.uploadResume(me.mergeCandidateCallback)(res);
            })
        },
        import: function (extractData) {
            var me = this;
            var bool = extractData.isOneClickSave;
            me.host = extractData.host;
            me.joborderId = extractData.joborderId;
            var callback = bool ? me.importOneClickDataCallBack : me.importCallback;
            return me.extracting(extractData).then(function (res) {
                return me.uploadResume(callback, extractData.bool)(res);
            });
        },
        //简历提取
        extracting: function(data) {
            var me = this;
            var additionalInfo;
            var promise = $.Deferred().resolve(me.base64_info);
            return me.getHTML().then(function (res) {
                var html = res;
                additionalInfo = {
                    html: [html],
                    external_id: data.id,
                    joborderId: data.joborderId,
                    candidateId: data.similarId,
                    note: data.note
                };
                return promise
            }).then(function(base64_info){
                $.extend(additionalInfo, base64_info);
                return additionalInfo;
            }).then(function(ret){
                return ret;
            });
        },

        //上传简历
        uploadResume: function (callback, method) {
            var me = this;
            return function(info){
                var joborderId = info.joborderId;
                var note = info.note;

                delete info['joborderId'];
                delete info['note'];

                // direct import candidate
                if (method) {
                    info['to_candidate'] = true;
                }

                return Utils.postJSON(URL.uploadresume,{
                    data : JSON.stringify(info)
                }).then(function(data) {
                    if (data.status) {
                        data['joborderId'] = joborderId;
                        data['note'] = note;
                        if (callback) {
                            callback.call(me, data);
                        }
                    } else {
                        if (method) {
                            callback.call(me, data);
                        } else {
                            var result = data.data || data.message;
                            if (result == 'login required') {
                                window.open(GLLUE_HOST);
                            }
                        }
                    }
                    return data;
                }, function(jqXHR, status, message) {
                    console.error(jqXHR.responseJSON.message || message);

                    if (method) {
                        callback.call(me, jqXHR.responseJSON.message || message);
                    }

                    return $.Deferred().resolve();
                });
            };
        }
    };
    window.extractor = APP;
})(window);