(function () {

    let modelMap = {
        candidate: {
            processData: function (data) {
                return data.map(function (item) {
                    return {
                        id: item.id,
                        text: '#' + item.id + ' ' + [item.__name__, item.client, item.title].filter(function (it) {
                            return !!it;
                        }).join(' - ')
                    };
                });
            },

            request: function (params, success, failure) {
                var data = {
                    ordering: '-id',
                    paginate_by: '50',
                    gql: '',
                    _key: params.data.term
                };

                var request = Utils.getJSON('/rest/candidate/list', data);

                request.then(function (res) {
                    var arr = res.list || [];
                    success(arr);
                });

                return request;
            },
        },
        joborder: {
            processData: function (data) {
                return JSON.parse(JSON.stringify(data)).map(function (item) {
                    return {
                        id: item.id,
                        text: '#' + item.id + ' ' + item.jobTitle + (item.client ? ' of ' + item.client.name : '') + (item.bu ? ' of '+  item.bu.name : '')
                    }
                });
            },
            request: function (params, success, failure) {
                var data = {
                    ordering: '-lastUpdateDate',
                    paginate_by: '50',
                    demandKeys: JSON.stringify(["bu", "lineManager", "client"]),
                    gql: $.param({
                        jobStatus: 'Live',
                        joborderuser_set__user__eq: '{{user.id}}',
                    }),
                    _key: params.data.term
                };

                var request = Utils.getJSON('/rest/joborder/list', data);
                request.then(function (res) {
                    var arr = res.list || [];
                    success(arr);
                });

                return request;
            }
        }
    };


    let tpl = `
        <select2 :processData="processData"
             :request="request"
             @inputChange="inputChange">
    </select2>
        
    `;
    Vue.component('simple-select', {
        template: tpl,
        props: ['model'],
        computed: {
            processData() {
                return modelMap[this.model].processData;
            },
            request() {
               return  modelMap[this.model].request;
            }
        },
        methods: {
            inputChange: function (val) {
                this.$emit('inputChange', val);
            }
        }

    });
})();
