/* global Config, chrome, utils, Logger, Tracker */

(function(root, $) {
    "use strict";
    var Utils = root.Utils;
    var assert = Utils.assert;

    var SEL_ROOT = '.fn-joborder';
    var SEL_TOOLBAR = '.gllue-toolbar';

    // internal variable
    var URL = root.Config.URL;

    function getParamFromQuery(key) {
        var values = location.search.slice('1').split('&')
            .reduce(function(curr, item) {
                var pair = item.split('=');
                curr[pair[0]] = pair[1];

                return curr;
            }, {});

        return values[key];
    }



    var App = {
        init: function(config) {
            var me = App;

            me.config = config;

            // use default style or not
            me._useDefault = !config.selectors || !config.selectors.panel;
            // set default selectors
            config.selectors = config.selectors || {};
            assert(config.selectors.submit, 'selector "submit" can not be undefined');

            // site name
            me.site = config.site;
            // selectors is used to match target site's form elements
            me.selectors = config.selectors;

            // whether need to mapping
            me.verify = config.verify || function() {return true;};
            // whether need to fill
            me.autofill = config.autofill || function() {return false;};

            me.simpleMapping = config.simpleMapping;


            if (!me.autofill()) {
                me.editMode = true;
                if (!config.unique_id) {
                    console.log('edit mode and hide toolbar!');
                    $('body').find(SEL_TOOLBAR).hide();
                    me.hidden = true;
                }
            }

            Utils.getUserContext().then(function(userContext) {
                if (userContext && userContext.user ) {
                    me.userEmail = userContext.user.email;
                }
            });
            if (!me.hidden) {
                me.addEvents();
            }
        },

        fillForm: function (form, usingNew) {
            var me = App;
            var mapping = me.selectors.form || {};
            var simpleMapping;
            var defaultFormFill;
            if (usingNew){
                mapping = me.selectors.newVersionForm;
                simpleMapping = me.selectors.simpleMapping;
                defaultFormFill = me.selectors.defaultFormFill;
            }

            if (!form || !me.autofill()) {
                return;
            }


            Object.keys(form).forEach(function(key) {
                var fieldValue = form[key];
                if (!fieldValue){
                    return;
                }
                if (!me.verify(key)) {
                    return;
                }

                if (mapping && mapping[key]) {

                    mapping[key].call(form, fieldValue);

                } else if (simpleMapping && simpleMapping[key]!== undefined) {
                    key = simpleMapping[key];
                    var sk = me.selectors.simpleMapping_selector_key || 'name';
                    $('['+sk+'="' + key + '"]').val(fieldValue);
                } else if (fieldValue) {
                    $('[name="' + key + '"]').val(fieldValue);

                }

            });

            if (defaultFormFill){
                defaultFormFill();
            }

            if (me.userEmail && me.selectors.email) {
                if (typeof(me.selectors.email) === 'function') {
                    me.selectors.email.call(me, me.userEmail);
                } else {
                    $(me.selectors.email).val(me.userEmail);
                }
            }
        },
        addEvents: function(){
            var me = App;

            // joborder list root element
            me.$root = $(SEL_ROOT)
                .addClass('gllue-p-' + me.site);

            // whether use default style
            if (me._useDefault) {
                me.$root.addClass('gllue-p-default');
            }
            var has_submit = $( me.selectors.submit).length;
            if (!has_submit){
                Logger.error('该招聘网站没有submit按钮的selector:'+me.config.site);
            }
            assert(has_submit, 'selectors.submit dom not exists');

            // when user click 'submit' in position page
            $( me.selectors.submit ).click(function() {
                me.clickedSubmit = true;
                chrome.runtime.sendMessage({
                    type: 'position/info',
                    data: me.getPositionInfo(),
                }, function(response) {

                });

            });

        },

        getPositionInfo: function() {
            var me = App;
            return {
                site: me.config.site,
                editMode: me.editMode,
                joborder_id: me.joborderId,
                page_url: window.location.href,
                html: document.all[0].innerHTML,
                tabId: getParamFromQuery('tabId')
            }

        },
        onJoborderIdChange: function (joborderId) {
            var me = this;
            me.joborderId = joborderId;

            if ( me._converting || me.editMode ) {
                return;
            }

            me._converting = true;

            var newUrl = "/rest/jobposting/form/<joborder>/convert_to_site_new?to=<site>"
                .replace('<joborder>', me.joborderId)
                .replace('<site>', me.site);

            var oldUrl = URL.convert_form
                .replace('<joborder>', me.joborderId)
                .replace('<site>', me.site);
            Utils
                .getJSON(newUrl)
                .then(function(form){
                    var usingNew = true;
                    if(form && form.status !== false) {
                        me.fillForm(form, usingNew);
                    }
                    me._converting = false;
                }, function(err){
                // If get New Url failed, try using old form
                    return Utils.getJSON(oldUrl)
                        .always( function onConvertForm(form) {
                            console.log('Using old form');
                            if(form && form.status !== false) {
                                me.fillForm(form);
                            }
                            me._converting = false;
                        });
                });

        },
        blockRequest: function (blockData, callback) {
            chrome.runtime.sendMessage({
                type: 'block/webrequest',
                data: blockData
            }, function(res) {
                console.log(res);
                if (callback) {
                    callback(res);
                }
            });
        },
    };

    root.Position = App;

}(window, jQuery));
