/* global Config, Logger, chrome, Tracker */
try {
    (function (window) {
        'use strict';
        var Utils = window.Utils;
        var Position = window.Position;

        var app = angular.module('gllue-toolbar', ['pascalprecht.translate', 'common-module', 'ui.select', 'ngSanitize']).config(['$translateProvider', function ($translateProvider) {
            $translateProvider.translations('en', window.translation.en);

            $translateProvider.translations('zh_CN', window.translation.zh_CN);

            $translateProvider
                .registerAvailableLanguageKeys(['en', 'zh_CN'], {
                    'en_US': 'en',
                    'en_UK': 'en',
                    'zh_TW': 'zh_CN',
                    'zh_HK': 'zh_CN'
                }).determinePreferredLanguage();

            $translateProvider.fallbackLanguage('en');
        }]).run([function () {
            Utils.draggableToolbar();
        }]);


        // filters
        app.filter('joborder', function () {
            return function (input) {
                if (!input) {
                    return;
                }

                return '#' + input.id + ' ' + input.jobTitle + (input.client ? ' of ' + input.client.name : '') + (input.bu ? ' of '+  input.bu.name : '');
            };
        });

        // controllers
        app.controller('WrapperCtr', ['$scope', '$rootElement', function ($scope, $rootElement) {
            $scope.joborderId = '';

            $scope.hide = false;
            $scope.markHide = false;
            $scope.toggle = function (mark) {
                if(mark){
                    $scope.markHide = !$scope.markHide;
                }
                else{
                    $scope.hide = !$scope.hide;
                }
            };

            if (window.Config.debug) {
                $scope.debug = true;
            }

            Utils.getGllueHost().done(function (res) {
                $scope.host = res;
            });

            $scope.joborders = [];

            Position.blockRequest(null, function (res) {
                $scope.siteBlocked = res.BLOCK_SITE_WEBREQUEST;
                $scope.sitesBlocked = res.BLOCK_ALL_WEBREQUEST;
            });

            $scope.refreshJoborders = function(jobTitle) {
                var gql = {};
                var data = {
                    ordering: '-lastUpdateDate',
                    paginate_by: '50',
                    demandKeys: JSON.stringify(["bu", "lineManager"])
                };
                if (jobTitle) {
                    gql['jobTitle__icontains'] = jobTitle;
                } else {
                    gql['jobStatus'] = 'Live';
                }

                return Utils.getJSON('/rest/user/context').then(function(context) {
                    gql['joborderuser_set__user__eq'] = context.user.id;
                    data.gql = $.param(gql);
                    return Utils.getJSON('/rest/joborder/list', data).then(function (res) {
                        $scope.$apply(function () {
                            if (res.status !== undefined && !res.status && res.message === 'login required') {
                                $scope.logout = true;
                            } else {
                                $scope.joborders = res.list;
                            }
                            $rootElement.removeClass('gllue-hide');
                        });
                    });
                });
            };

            Position.setJoborderId = function(joborderId){
                if (joborderId) {
                    //$scope.refreshJoborders(joborderId, true);
                    Utils.getJSON('/rest/joborder/list', {
                        id: joborderId,
                        demandKeys: JSON.stringify(["bu", "lineManager"])
                    }).then(function (res) {
                        $scope.$apply(function () {
                            if (res.status !== undefined && !res.status && res.message === 'login required') {
                                $scope.logout = true;
                            } else {
                                $scope.selectJoborder(res.list[0]);
                                $scope.joborder = res.list[0];
                            }
                        });
                    });
                }
            };

            $scope.selectJoborder = function (selected) {
                $scope.joborderId = selected.id;

                Position.onJoborderIdChange($scope.joborderId);
            };

            $scope.blockSite = function ($event) {
                $event.preventDefault();
                Position.blockRequest({site: true}, function (res) {
                    $scope.$apply(function () {
                        $scope.siteBlocked = res.BLOCK_SITE_WEBREQUEST;
                    });
                });
            };

            $scope.blockAllSites = function ($event) {
                $event.preventDefault();
                Position.blockRequest({all: true}, function (res) {
                    $scope.$apply(function () {
                        $scope.sitesBlocked = res.BLOCK_ALL_WEBREQUEST;
                    });
                });
            };
            $scope.manageSubscribe = function(noteType){
                var opt = {
                    'searchId': $scope.currentSubscribe && $scope.currentSubscribe.search_id,
                    'noteType': noteType
                };
                Utils.showSearchResultPage(opt);
            };

        }]);

        // bootstrap gllue-toolbar in document
        angular.bootstrap(document.getElementById('gllue-toolbar'), ['gllue-toolbar']);
    }(window));
} catch(err) {
    Logger.error(err);
}
