/* global Logger */
(function(root, $) {
    "use strict";
try{
    var Utils = root.Utils;
    var config = root.Config;
    var translation = root.translation;
    var translate_key = ({
        'en-US': 'en',
        'en-UK': 'en',
        'zh-TW': 'zh_CN',
        'zh-HK': 'zh_CN',
        'zh-CN': 'zh_CN'
    })['zh-CN'];
    var trans_table = translation[translate_key];
    var App = {
        init: function(config) {
            var me = App;
            me.selectors = config.selectors;
            me.site = config.site;
            me.th_note = config.th_note || $("<th></th>");
            me.th_visited = config.th_visited || $("<th></th>");
            me.empty_td = config.empty_td || $('<td></td>');
            me.td_note = config.td_note || '<td></td>';
            me.td_visited = config.td_visited || '<td></td>';
        },

        append_note: function(resume_note_list){
            var me = App;

            // Prepend suitable tr
            me.find_tr().each(function(){
                var $this = $(this);
                if (me.need_to_append_note($this)){
                    var unique_id = me.find_unique_id_from_tr($this);
                    var note_info = resume_note_list[unique_id] || {};
                    var note_str = note_info.note || config.NOTE_TYPE.DEFAULT;
                    var visited = note_info.visited;
                    var td_to_append = me.build_td_to_append(unique_id, note_str);
                    var popover_content = me.build_popover_content(unique_id, td_to_append);
                    me.add_popover(td_to_append, popover_content);
                    $(this).prepend(td_to_append);

                    var visited_td = me.build_visited_to_append(unique_id, visited);
                    $this.prepend(visited_td);
                } else if (me.need_to_append_empty($(this))){
                    $this.prepend(me.empty_td.clone());
                    $this.prepend(me.empty_td.clone());
                }

            });

            // IF there is any error before, th should not be prepend, or the origin page will be strange
            // Prepend th
            me.append_th();
        },
        append_th: function(){
            var me = App;
            var callback = me.before_append_th_callback;
            if (callback) {
                callback();
            }
            me.find_th().prepend(me.th_note);
            me.find_th().prepend(me.th_visited);
        },
        need_to_append_note : function(){
            return true;
        },
        need_to_append_empty : function($this){
            return !this.need_to_append_note($this);
        },
        find_unique_id_from_tr : function($this){
            var me = App;
            var resume_div = $this.find(me.selectors.resume_div);
            return me.find_unique_id(resume_div);
        },
        build_td_to_append : function(unique_id, note_str){
            var td = $(this.td_note);
            td.text(trans_table[note_str]);

            td.on('resumeNoteChanging', function(event, id){
                if (id == unique_id ){
                    td.text(trans_table['modifying']);
                }
            });
            td.on('resumeNoteChanged', function(event, id, note){
                if (id == unique_id){
                    td.text(trans_table[note]);
                }
            });
            return td;
        },
        build_popover_content : function(unique_id, td){
            var me = App;
            var build_button= function(type, button_class, callback){
                var button = $('<button></button>');
                if (button_class) {
                    button.addClass(button_class);
                }
                button.on('click', function(){
                    if (callback) {
                        callback();
                    }
                    if (td) {
                        td.trigger("resumeNoteChanging", [unique_id]);
                    }
                    Utils.saveNote(type, me.site, unique_id).then(function(){
                        if (td){
                            td.trigger('resumeNoteChanged', [unique_id, type]);
                        }
                    }, function(err){
                        Logger.error(err);
                    });
                });
                button.text(trans_table[type]);
                return button;
            };
            var div = $('<div class="gllue-resume-note-button-popover"></div>');
            div.append(build_button(config.NOTE_TYPE.CONSIDER, 'btn btn-small'));
            div.append(build_button(config.NOTE_TYPE.DECLINE, 'btn btn-small'));
            return div;
        },
        build_visited_to_append : function(unique_id, visited){
            var td = $(this.td_visited);
            if (visited){
                td.text(trans_table['been_visited']);
            }
            return td;
        },
        add_popover: function($this, popover_content){
            $this.poshytip({
                className : 'tip-glluepopover',
                content : popover_content,
                alignX : 'center'
            });
        },
        get_unique_id_list : function(){
            var me = App;
            return $(me.selectors.resume_div).toArray().map(function(each){
                return me.find_unique_id($(each));
            });
        },
        get_resume_note_list : function(){
            var me = App;
            var unique_id_list = me.get_unique_id_list();
            return Utils.getNotes(unique_id_list);
        },
        get_and_append_resume_note : function(){
            var me = App;
            me.get_resume_note_list().then(function(ret){
                me.append_note(ret);
            }, function(err){
                Logger.error(err);
            });
        }
    };
    root.ResumeNote = App;
} catch(err){
    Logger.error(err);
}
}(window, jQuery));
