/* global chrome */

(function(root, $){
    'use strict';
    var Utils = root.Utils;
    var ctx = chrome.extension.getBackgroundPage();
    var Config = ctx.Config;
    Utils.draggableToolbar();

    var app = angular.module('gllue-search-result', ["ui.router"]);
    function getRemoteUrl($http, path){
        var host = Config.HUB_HOST;
        var url = host + path;
        return $http({
            url : url,
            method: 'GET'
        }).then(function(response){
            return response.data;
        });
    }

    app.config(function($stateProvider, $urlRouterProvider){
        $urlRouterProvider.otherwise('/manageSubscribe');
        $stateProvider
            .state('manageSubscribe', {
                url: '/manageSubscribe',
                controller: 'SubscribeSearchManagerCtrl',
                templateProvider: function($http){
                    return getRemoteUrl($http, '/assets/templates/search-subscribe/manager-subscribe.html');
                }
            })
            .state('showSearchResult',{
                url: '/showSearchResult?search_id',
                controller : 'SearchResultCtrl',
                templateProvider: function($http){
                    return getRemoteUrl($http, '/assets/templates/search-subscribe/show-search-result.html');
                }
            });

    });

    app.controller('SubscribeSearchManagerCtrl', ['$scope','$q', '$state', function($scope, $q, $state){
        $scope.subscribes = [];

        $scope.success = "";
        $scope.recentDeleted = {};

        function deleteSubscribeFromArray(search_id){
            for (var idx in $scope.subscribes){
                if ($scope.subscribes[idx].search_id == search_id){
                    $scope.subscribes.splice(idx, 1);
                    $scope.success = "删除订阅成功";
                    return;
                }
            }
        }

        $q.when(Utils.subscribeList()).then(function(list){
            $scope.subscribes = list;
        }, function(err){
            console.log(err);
        });

        $scope.startSearch = function(){
            var list = $scope.subscribes.map(function(each){
                return {
                    'id': each.search_id,
                    'url': each.url
                };
            });
            $q.when(Utils.startSearch(list)).then(function(){
                $scope.success = "已开始搜索";
            },function(err){

            });
        };

        $scope.cancelSubscribe = function(search){
            $q.when(Utils.subscribe('cancel_subscribe', {search_id : search.search_id})).then(function(data){
                console.log(data);
                $scope.recentDeleted = search;
                deleteSubscribeFromArray(search.search_id);
            }, function(err){
                console.log(err);
                $scope.recentDeleted = search;
                deleteSubscribeFromArray(search.search_id);
            });
        };

        $scope.getSearchResult = function(search){
            chrome.runtime.sendMessage({
                type: 'redirectToSearch',
                redirectInfo: search
            });
        };

    }]);

    app.controller('SearchResultCtrl', ['$scope', '$q', '$state','$sce', function($scope, $q, $state, $sce){

        $scope.search = {};
        $scope.search_id = $state.params.search_id;
        $q.when(Utils.subscribe('search_detail', {
            search_id : $scope.search_id
        })).then(function(search){
            $scope.search=search;
        }, function(err){
            console.log('Error in get search_info');
            console.log(err);
        });


        $q.when(Utils.getSearchResult({search_id: $scope.search_id})).then(function(result){
            $scope.searchResult = result.list;
        }, function(err){
            console.log("Error in SearchResultCtrl");
            console.log(err);
        });

        $scope.buildRecentUpdate = function(recentUpdate){
            var p = $("<p></p>");
            recentUpdate.forEach(function(char){
                var span = $ ("<span class='color-span'></span>");
                if (parseInt(char, 10) === 0) {
                    span.addClass('red-span');
                } else {
                    span.addClass('green-span');
                }
                p.append(span);
            });
            return $sce.trustAsHtml(p.html());
        };

    }]);

    angular.bootstrap(document.getElementById('result-main-div'), ['gllue-search-result']);
}(window, jQuery));
