/* global chrome, Logger */

try{
(function(root, $){
    'use strict';
    var Utils = root.Utils;
    var Logger = root.Logger;
    var app = angular.module('gllue-search-subscribe', ['common-module']);
    Utils.draggableToolbar();

    app.controller('NaiveCtrl', ['$rootElement', '$scope', 'modals', '$q', function($rootElement, $scope, modals, $q){
        $scope.hideSearch=false;
        $scope.savedName = "";
        $scope.hasSubscribed = false;
        $scope.toggleSearchBar = function(){
            $scope.hideSearch = !$scope.hideSearch;
        };
        $scope.checkSubscribeSuccess = false;
        Utils.getConfigValue().done(function(config){
            console.log('configvalue from gllueweb: ', config);
            if(config.enable_search_subscribe) {
                var curDomain = Utils.getDomain();
                if ( curDomain=== '51job') {
                    check51jobSearchList();
                } else if (curDomain === 'liepin') {
                    checkLiepinSearchList();
                }

                chrome.runtime.sendMessage({
                    type: 'getCurrentRequest'
                }, function(req){
                    function formDataChange(request){
                        if (request.formData){
                            var commaJoined = {};
                            var key;
                            for (key in request.formData){
                                commaJoined[key] = request.formData[key].join(',');
                            }
                            var equalJoined = [];
                            for (key in commaJoined){
                                var val = commaJoined[key];
                                equalJoined.push([key, val].map(encodeURIComponent).join('='));
                            }
                            request.formData = equalJoined.join('&');
                        }
                        return request;
                    }
                    $scope.currentRequest = formDataChange(req);
                    checkSubscribe();
                });
            }
        });

        function get51IdList() {
            var unique_ids = [];
            $(document).find('.resume_p1 .SearchR>a').each(function(){
                unique_ids.push($(this).text());
            });
            console.log('unique_ids', unique_ids);
            return unique_ids;
        }
        function getLiepinUid(el) {
            var mat = el.href.match(/res_id_encode=(\w+)/);
            if (mat && mat.length === 2){
                return mat[1];
            }
        }
        function getLiepinIdList() {
            var unique_ids = [];
            var $target = $(document).find('.resume-search a[data-selector="link-resume"]');
            $target.each(function(){
                var uid = getLiepinUid(this);
                if(uid){
                    unique_ids.push(uid);
                }
            });
            console.log('unique_ids', unique_ids);
            return unique_ids;
        }
        var targetColor = '#F89406';
        function checkLiepinSearchList() {
            console.log('checkLiepinSearchList');
            var unique_ids = getLiepinIdList();
            Utils.postJSON('/rest/jobposting/check_sim_resume_uniques', {
                data: JSON.stringify({
                    site: Utils.getDomain(),
                    unique_ids: unique_ids
                })
            }).done(function(res){
                console.log('Liepin search list resumes: ', res);
                if (res && typeof(res) === 'object') {
                    var keys = Object.keys(res);
                    $(document).find('.resume-search a[data-selector="link-resume"]').each(function(){
                        var uid = getLiepinUid(this);
                        if(uid && keys.indexOf(uid) >= 0) {
                            var targetSite = res[uid];
                            $(this).css('background-color', targetColor);
                            var origin = $(this).find('strong').text();
                            $(this).find('strong').text('(重)'+origin);
                        }
                    });
                }
            });
        }
        function check51jobSearchList() {
            console.log('check51jobSearchList');
            var unique_ids = get51IdList();
            Utils.postJSON('/rest/jobposting/check_sim_resume_uniques', {
                data: JSON.stringify({
                    site: Utils.getDomain(),
                    unique_ids: unique_ids
                })
            }).done(function(res){
                console.log('51job search list resumes: ', res);
                if (res && typeof(res) === 'object') {
                    var keys = Object.keys(res);
                    $(document).find('.resume_p1 .SearchR>a').each(function(){
                        var uid = $(this).text();
                        if(uid && keys.indexOf(uid) >= 0) {
                            var targetSite = res[uid];
                            $(this).css('background-color', targetColor);
                            $(this).text('(重)'+uid);
                        }
                    });
                }
            });
        }

        function checkSubscribe(){
            $q.all([Utils.subscribe('check_subscribe', $scope.currentRequest), Utils.getJSON('/rest/jobposting/list_subscribe')]).then(function (response){
                $scope.subscribes = response[1].list;

                if (response[0] && response[0].name){
                    $scope.savedSubscribe = response[0];
                    $scope.hasSubscribed = true;

                    for (var i = 0, l = $scope.subscribes.length; i < l; i++) {
                        if ($scope.savedSubscribe.id === $scope.subscribes[i].search_id) {
                            $scope.currentSubscribe = $scope.subscribes[i];
                            break;
                        }
                    }
                }

                $scope.checkSubscribeSuccess = true;

                $rootElement.removeClass('gllue-hide');
            }, function (err) {
                console.log('check_subscribe failed');
                Logger.error(err);
                console.log('Hide toolbar');
                $scope.checkSubscribeSuccess = true;
            });
        }

        $scope.subscribeSearch = function(){
            var request = $scope.currentRequest;
            request.name = $scope.savedName;
            request.html = document.all[0].innerHTML;
            $q.when(Utils.subscribe('subscribe_search', request)).then(function(data){
                checkSubscribe();
            }, function(err){
                Logger.error(err);
            });
        };

        $scope.manageSubscribe = function(noteType){
            var opt = {
                'searchId': $scope.currentSubscribe && $scope.currentSubscribe.search_id,
                'noteType': noteType
            };
            Utils.showSearchResultPage(opt);
        };
    }]);

    // bootstrap
    angular.bootstrap(document.getElementById('gllue-searchbar'), ['gllue-search-subscribe']);
}(window, jQuery));
}catch(err){
    Logger.error(err);
}
