/* global Logger */
try {
    (function (window) {
        'use strict';
        var Logger = window.Logger;

        // error handling
        window.onerror = function () {
            Logger.onerror.apply(window, Array.prototype.slice.call(arguments));
        };

        var app = angular.module('common-module', []);

        // angular exception
        app.factory('$exceptionHandler', function () {
            return function (exception, cause) {
                Logger.onerror(exception.message, null, null, null, exception);
            };
        });

        // services
        app.service("modals", ['$rootScope', '$q', function ($rootScope, $q) {
            var modal = {
                deferred: null,
                params: null
            };

            function open(type, params, pipeResponse) {
                var previousDeferred = modal.deferred;
                modal.deferred = $q.defer();
                modal.params = params;

                if (previousDeferred && pipeResponse) {
                    modal.deferred.promise
                        .then(previousDeferred.resolve, previousDeferred.reject);
                } else if (previousDeferred) {
                    previousDeferred.reject();
                }
                $rootScope.$emit("modals.open", type);
                return (modal.deferred.promise);
            }

            function params() {
                return (modal.params || {});
            }

            function proceedTo(type, params) {
                return (open(type, params, true));
            }

            function reject(reason) {
                if (!modal.deferred) {
                    return;
                }
                modal.deferred.reject(reason);
                modal.deferred = modal.params = null;
                $rootScope.$emit("modals.close");
            }

            function resolve(response) {
                if (!modal.deferred) {
                    return;
                }
                modal.deferred.resolve(response);
                modal.deferred = modal.params = null;
                // Tell the modal directive to close the active modal window.
                $rootScope.$emit("modals.close");
            }

            // Return the public API.
            return ({
                open: open,
                params: params,
                proceedTo: proceedTo,
                reject: reject,
                resolve: resolve
            });
        }]);

        app.directive("gllueModals", ['$rootScope', 'modals', function ($rootScope, modals) {
            function link(scope, element) {
                scope.subview = null;
                element.on(
                    "click",
                    function handleClickEvent(event) {
                        if (element[0] !== event.target) {
                            return;
                        }
                        scope.$apply(modals.reject);
                    }
                );
                $rootScope.$on("modals.open",
                    function handleModalOpenEvent(event, modalType) {
                        element.removeClass('gllue-hide');
                        scope.subview = modalType;
                    }
                );
                $rootScope.$on("modals.close",
                    function handleModalCloseEvent(event) {
                        element.addClass('gllue-hide');
                        scope.subview = null;
                    }
                );
            }

            return link;
        }]);

        window.Config = window.Config || {};

        window.Config['NOTE_TYPE'] = {
            DECLINE : 'decline',
            CONSIDER : 'consider',
            DEFAULT : 'not_be_noted',
            ERROR : 'resume_note_error'
        };

    }(window));
} catch (err) {
    Logger.error(err);
}
