/* global $, chrome, Config, Mustache, Utils, pako, Logger */

/*
* require: jquery, shared/config
* */
try{

(function(root) {
    'use strict';
    var HOST_NAME = 'gllueHost';
    var GLLUE_HOST;

    var RE_MAIMAI_CHAT = /^http(s)?:\/\/maimai\.cn\/im\//i;

    var Logger = root.Logger;

    chrome.storage.onChanged.addListener(function(changes, namespace) {
        var hostChange = changes[HOST_NAME];

        if (hostChange && hostChange.newValue !== hostChange.oldValue) {
            GLLUE_HOST = hostChange.newValue;
            window.GLLUE_HOST = hostChange.newValue;
        }
    });

    root.Utils = {
        /**
         *
         * @returns {Promise}
         */
        getGllueHost: function() {
            var dfd = $.Deferred();

            if(GLLUE_HOST) {
                return dfd.resolve(GLLUE_HOST);
            }

            chrome.storage.sync.get(HOST_NAME, function(items) {
                GLLUE_HOST = items[HOST_NAME];

                dfd.resolve(items[HOST_NAME]);
            });

            return dfd;
        },

        gLocalStorage: function (key, value) {
            if (value && typeof value === 'string') {
                var obj = {};
                obj[key] = value;
                chrome.storage.sync.set(obj, function () {});
            } else if (typeof value === 'function') {
                var callback = value;
                chrome.storage.sync.get(key, callback);
            }
        },

        // tiny assert function
        assert: function assert(value, desc) {
            Logger.assert(value, desc);
        },

        ajaxFromBackground: function(config) {
            var deferred = $.Deferred();

            var action = config.type || 'get';
            var url = config.url;
            var data = config.data || '';

            if (action === 'get') {
                if (url.indexOf('?') > -1) {
                    url += '&' + $.param(data);
                } else {
                    url += '?' + $.param(data);
                }
            }

            chrome.runtime.sendMessage({
                type: 'ajax',
                action: action,
                url: url,
                data: data
            }, function(response) {
                deferred.resolve(response);
            });

            return deferred;
        },

        /**
         * helper for get json by 'host' and 'url'
         * @param url
         * @returns {Promise}
         */
        getJSON: function(url, data) {
            var deferred;
            // For security reason, when location protocol is 'https:' should send ajax request from background.
            if(window.location.protocol === 'https:') {

                deferred = Utils.ajaxFromBackground({
                    url: url,
                    data: data
                });

            } else {
                deferred = $.Deferred();
                Utils.getGllueHost().then(function(host) {
                    $.getJSON(host + url, data).then(deferred.resolve, deferred.reject);
                });
            }

            return deferred;
        },

        postJSON: function(url, data, options) {
            var deferred;

            options = options || {};

            // For security reason, when location protocol is 'https:' should send ajax request from background.
            if(window.location.protocol === 'https:') {

                deferred = Utils.ajaxFromBackground($.extend({
                    url: url,
                    type: 'post',
                    data: data
                }, options), deferred);

            } else {
                deferred = $.Deferred();
                Utils.getGllueHost().then(function (host) {
                    $.post(host + url, data)
                        .then(deferred.resolve, deferred.reject);
                });
            }

            return deferred;
        },

        deflate: function(rawString) {
            return window.btoa(pako.deflate(rawString, { to: 'string' }));
        },

        /**
         * Get template string by url path
         * @param path
         * @returns {Promise}
         */
        getTplStr: function(path, data) {
            console.warn('This api should be changed. We are about to give up Mustache template engine.');
            var dfdTpl = $.Deferred();
            if(window.location.protocol === 'https:') {
                //TODO: use HUB_HOST from background;
                Utils.ajaxFromBackground({
                    url: path,
                    type: 'get'
                }, function(tpl){
                    dfdTpl.resolve(Mustache.render(tpl, data));
                });
            } else {
                $.get(root.Config.HUB_HOST + path).done(function(tpl) {
                    dfdTpl.resolve(Mustache.render(tpl, data));
                });
            }

            return dfdTpl;
        },

        /**
         * eg: getUserContext().then(function(userContext) { ... });
         * @returns {*}
         */
        getUserContext: function getUserContext() {
            return new Promise(function(resolve, reject) {
                chrome.runtime.sendMessage({
                    type: 'user/context'
                }, resolve);
            });
        },

        getCookie: function getCookie(site) {
            return new Promise(function(resolve, reject){
                chrome.runtime.sendMessage({
                    type: 'site/cookie',
                    data: {
                        site: site,
                        url: window.location.href
                    }
                }, resolve);
            });
        },

        getHTML: function(deflate) {
            var $html = $('html').clone();
            var html;
            var href = window.location.href;

            $html.find('.gllue-toolbar').remove();

            if (RE_MAIMAI_CHAT.test(href)) {
                var $temp = $($('iframe:last').contents()[0]).find('html');
                if ($temp.length) {
                    $html = $temp.clone();
                }
            }

            var domain = this.getDomain();
            switch (domain) {
                case "liepin":
                case "lietou":
                    $html.find('img').remove();
                    $html.find('link[type="image/x-icon"]').remove();
                    break;
                default:
                    break;
            }

            html = $html[0].outerHTML;
            if (deflate) {
                return Utils.deflate(html);
            } else {
                return html;
            }
        },

        /**
         * Get template string by url path
         * @returns {Promise}
         */
        checkDupe: function(type) {
            var me = this;
            var deferred = $.Deferred();
            var domain = this.getDomain();
            var p = $.Deferred().resolve({});
            var URL = {
                'extactly': '/rest/candidate/check_exists/',
                'fuzzy': '/rest/candidate/check_html_content'
            };
            type = type || 'extactly';

            var url = URL[type];

            switch (domain) {
                case "liepin":
                case "lietou":
                    //p = utils.getBase64();
                    break;
                case 'lagou':
                    // lagou 暂时不支持简历查重
                    deferred.resolve({
                        status: true
                    });
                    return p;
                    break;
                default:
                    break;
            }
            p.then(function(data){
                data.html = Utils.getHTML(true);
                // console.log(JSON.stringify(data));
                Utils.postJSON(url, {
                    data: JSON.stringify(data)
                }).then(function (dupRes) {
                    //如果候选人没有权限自动共享
                    if (dupRes.data.candidate && dupRes.data.candidate.length &&  dupRes.data.candidate[0].islimited) {
                        var dupInfo = dupRes.data.dup_info[0];
                        var requestData = {};
                        requestData[dupInfo.type.toLowerCase()] = dupInfo.value;
                        Utils.postJSON('/rest/candidate/autoshare', {
                            data: JSON.stringify(requestData)
                        }).then(function (res) {
                            if (res.status) {
                                Utils.getJSON('/rest/candidate/' + res.data).then(function (res) {
                                    dupRes.data.candidate[0] = res;
                                    deferred.resolve(dupRes);
                                });
                            } else {
                                deferred.resolve(dupRes);
                            }

                        });
                    } else {
                        deferred.resolve(dupRes);
                    }

                }, deferred.reject);
            });

            return deferred;
        },

        getLagouResume: function () {
            var me = this;
            var dfd = $.Deferred();

            // new js api
            // ref: http://stackoverflow.com/questions/901115/how-can-i-get-query-string-values-in-javascript
            var searchParams = new URLSearchParams(window.location.search.slice(1));
            var resumeId = searchParams.get('resumeId');

            chrome.runtime.sendMessage({
                type: 'uploadLagouResume',
                resumeId: resumeId
            }, function(response) {
                console.log(response);
                dfd.resolve(response);
            });

            return dfd.promise();
        },

        getProfileId: function(domain){
            domain = domain || this.getDomain();
            var profileID = null;
            var test;
            switch (domain) {
                case "51job":
                    var url = window.location.search;
                    var text = $('body').text().trim();
                    if (/hidUserID/ig.test(url)) {
                        profileID = url.replace(/.*?hidUserID=(\d*)&.*/ig, '$1');
                    } else {
                        test = text.match(/\(ID:\d*\)/igm) || '';
                        profileID =  test.length ? test[0] : '';
                        profileID = profileID.replace(/\D/ig, '');
                    }
                    break;
                case "zhaopin":
                    profileID = $('.resume-left-tips-id:eq(0)').text().trim().replace(/^ID:(.*)$/gm, "$1");
                    break;
                case "highpin":
                    test = $('.resume-num').text().match(/\w+/g);
                    profileID = test.length >= 2 ? test[1] : '';
                    break;
                case "liepin":
                case "lietou":
                    profileID = $('.resume-info p, .resume-sub-info').length ?
                        $('.resume-info p, .resume-sub-info').text().trim().replace(/^.*(\d{8}).*(?:.|\s)+.*$/gm, "$1") : '';
                    break;
                case "linkedin":
                    profileID = '';
                    if ($('div.masthead').length) {
                        profileID = $('div.masthead').attr('id').replace('member-', '');
                    } else {
                        if ($('html').html().search(/memberId:"(\d+)"/gm)) {
                            profileID = RegExp.$1;
                        }
                    }
                    profileID = profileID.replace('li_', '');

                    // if empty maybe it's recruiter account
                    if (!profileID) {
                        profileID = $('html').find('input[name="profile-id"]').val();
                    }
                    break;
                case 'buildhr':
                    profileID = $('.print_content h1 span').text().replace(/^\D*(\d*).*/gm, '$1');
                    break;
                case 'yingjiesheng':
                    var _link = $("#statusdiv a:first").attr('href');
                    test = _link.match(/rid=(\d+)/) || [];
                    profileID =  test.length ? test[0] : '';
                    break;
                case 'lagou':
                    test = location.search.match(/deliverId=(\d+)/) || [];
                    profileID =  test.length ? test[0] : '';
                    break;
                case '58':
                    test = location.search.match(/rid=(\d+)/) || [];
                    profileID =  test.length ? test[0] : '';
                    break;
                default:
                    break;
            }
            return profileID;
        },

        getDomain: function(){
            if (window.location.protocol.indexOf('chrome') === 0){
                return null;
            }
            var hostname = window.location.hostname;
            var domain =  hostname.replace(/^(.*?)\.(.*?)\..*/g, "$2");
            if (domain == 'liepin'){
                if (hostname.indexOf('lpt') === 0){
                    domain  = 'liepin_hr';
                }
            }
            if (domain === 'jobmd'){
                domain = 'dxy';
            }
            return domain;
        },
        /*
            @params
            data: {id: id, note: note, joborderId: joborderId}
         */
        extracting: function(data) {
            var me = this;
            var candidateId = $('.gllue-candidate').data('id');
            var domain = this.getDomain();
            var html = Utils.getHTML();

            var additionalInfo = {
                html: [html],
                external_id: data.id,
                joborderId: data.joborderId,
                candidateId: candidateId,
                note: data.note
            };
            var p = Promise.resolve({});
            return p.then(function(base64_info){
                $.extend(additionalInfo, base64_info);
                return additionalInfo;
            }).then(function(ret){
                return ret;
            });
        },



        uploadResume: function (callback, method) {
            return function(info){
                var joborderId = info.joborderId;
                var note = info.note;

                delete info['joborderId'];
                delete info['note'];

                // direct import candidate
                if (method) {
                    info['to_candidate'] = true;
                }

                return Utils.postJSON("/rest/file/htmlfile",{
                        data : JSON.stringify(info)
                }).then(function(data) {
                    if (data.status) {
                        data['joborderId'] = joborderId;
                        data['note'] = note;
                        if (callback) {
                           callback(data);
                        }
                    } else {
                        if (method) {
                            callback(data);
                        } else {
                            var result = data.data || data.message;
                            if (result == 'login required') {
                                window.open(GLLUE_HOST);
                            }
                        }
                    }
                    return $.Deferred().resolve();
                }, function(jqXHR, status, message) {
                    console.error(jqXHR.responseJSON.message || message);

                    if (method) {
                        callback(jqXHR.responseJSON.message || message);
                    }

                    return $.Deferred().resolve();
                });
            };
        },
        openAddCandidate: function (data) {
            data.data = data.data.replace('/candidate#add','/crm#candidate/add');
            var joborderId = data.joborderId;
            var note = data.note;
            var url = GLLUE_HOST + data.data +
                    (joborderId ? '&joborderId=' + joborderId : '') +
                    (note ? '&note=' + note : '');
            window.open(url);
        },

        openCandidate: function (id) {
            var url = GLLUE_HOST + '/crm#candidate/detail?id=' + id;
            window.open(url);
        },
        responseToBase64: function(response) {
            var uInt8Array = new Uint8Array(response);
            var i = uInt8Array.length;
            var binaryString = new Array(i);
            while (i--) {
                binaryString[i] = String.fromCharCode(uInt8Array[i]);
            }
            var data = binaryString.join('');
            var base64 = window.btoa(data);
            return base64;
        },

        getImg: function(el){
            var deferred = $.Deferred();
            var x = new XMLHttpRequest();
            // x.onload = onload_func;
            x.onload = function() {
                deferred.resolve(Utils.responseToBase64(this.response));
            };

            x.responseType = 'arraybuffer';
            var $el = $(el);
            if ($el.length) {
                var src = $el.attr('src');
                x.open('GET', src, true);
                x.send();
            } else {
                deferred.resolve();
            }
            return deferred;
        },

        getBase64: function(){
            var getImgBase64 = function(el){
                return Utils.getImg(el);
            };
            var getTel = getImgBase64('img.telphone');
            var getEmail = getImgBase64('img.email');
            return $.when(getTel, getEmail).then(function (mobile, email) {
                return {
                    'mobile_base64' : mobile,
                    'email_base64' : email
                };
            });
        },

        addToJob: function(joborderId, data) {
            console.warn('This api is deprecated. Please use Utils.postJSON.');
            return Utils.postJSON(
                Config.url.replace("<joborderId>", joborderId),
                { data: JSON.stringify(data) }
            );
        },

        postNote: function (data) {
            return Utils.postJSON(Config.URL.add_note, {
                data: JSON.stringify(data)
            });
        },

        checkPostResult: function(fetchResult){
            fetchResult = fetchResult || function(res){
                return res;
            };
            return function(res){
                if (res.status!==false){
                    return $.Deferred().resolve(fetchResult(res));
                } else {
                    var err = res.message || 'INTERNAL_SERVER_ERROR';
                    console.log("Error In Response");
                    Logger.error(err);
                }
            };
        },

        saveNote: function(note, site, unique_id){
            site = site || this.getDomain();
            unique_id = unique_id || this.getProfileId(site);
            return this.getUserContext().then(function(user_context){
                //TODO: Andy
                var user_id = user_context.user.id || 1;
                var http_params = {
                        site : site,
                        user_id : user_id,
                        unique_id : unique_id,
                        modify_if_exist : 1
                    };
                var url = "/rest/jobposting/set_note_for_resume?" + $.param(http_params);
                var data = {
                        data : JSON.stringify({
                            note : note
                        })
                };
                return Utils.postJSON(url, data).then(Utils.checkPostResult());
            });
        },
        visitResume: function(){
            var site = this.getDomain();
            var href = window.location.href;
            var unique_id = this.getProfileId(site);
            return this.getUserContext().then(function(user_context){
                var user_id = user_context.user.id || 1;
                // TODO: Andy
                var http_params = {
                        site : site,
                        user_id : user_id,
                        unique_id : unique_id,
                        url : href
                    };
                var url = "/rest/jobposting/visit_resume?" + $.param(http_params);
                return Utils.postJSON(url).then(Utils.checkPostResult());
            });
        },

        getNotes: function(uniqueIdList){
            var site = this.getDomain();
            return this.getUserContext().then(function(userContext){
                var user_id = 1;
                try{
                    user_id = userContext.user.id;
                }catch(err){
                    user_id = 1;
                }
                // TODO: Andy
                var http_params = {
                    site : site,
                    user_id : user_id
                };
                var url = "/rest/jobposting/get_notes_for_unique_id_list?" + $.param(http_params);
                var data = {
                    data : JSON.stringify({
                        unique_id_list : uniqueIdList
                    })
                };
                return Utils.postJSON(url, data).then(Utils.checkPostResult());
            });
        },

        subscribe: function(action, search){
            var allowedActiones = ['subscribe_search',
                             'check_subscribe',
                             'change_subscribe_name',
                             'cancel_subscribe',
                             'search_detail'];
                if (allowedActiones.indexOf(action) < 0){
                console.log(action);
                throw 'Action Not Allowed'+action;
            }
            var site = this.getDomain();
            return this.getUserContext().then(function(userContext){
                var user_id = userContext.user.id || 1;
                //TODO: Andy
                var http_params = {
                    action : action,
                    user_id: user_id
                };
                if (site){
                    http_params.site = site;
                }
                var data = {
                    data : JSON.stringify(search)
                };

                var url = "/rest/jobposting/subscribe?" + $.param(http_params);
                return Utils.postJSON(url, data).then(Utils.checkPostResult());
            });
        },

        subscribeList: function(){
            return this.getUserContext().then(function(userContext){
                var user_id = userContext.user.id || 1;
                //TODO: Andy
                var http_params = {
                    user_id: user_id
                };
                var url = "/rest/jobposting/list_subscribe?" + $.param(http_params);
                return Utils.postJSON(url).then(Utils.checkPostResult());
            });
        },

        showSearchResultPage: function(opt){
            opt = $.extend({}, opt, {
                type: 'showSearchResult'
            });
            chrome.runtime.sendMessage(opt, function(response) {

            });
        },
        getSearchResult: function(searchResultReq){
            return this.getUserContext().then(function(userContext){
                var user_id = userContext.user.id || 1;
                //TODO: Andy
                var http_params = {
                    user_id: user_id
                };
                var url = "/rest/jobposting/get_search_result?" + $.param(http_params);
                var data = {
                    data : JSON.stringify({
                        search_result_req : searchResultReq
                    })
                };
                return Utils.postJSON(url, data).then(Utils.checkPostResult());
            });
        },
        startSearch: function(subscribeList){
            var url = "/rest/jobposting/search_manual";
            var data = {
                data: JSON.stringify({
                    search_list: subscribeList
                })
            };
            return Utils.postJSON(url, data).then(Utils.checkPostResult());
        },
        draggableToolbar: function () {
            var x = 0, y = 0;
            var innerX, innerY;
            var $el = $('.gllue-toolbar').css({'top': 80, 'right': 20});
            var $body = $('body');

            var $drag = $el.find('.gllue-drag-handle');
            $drag.css('cursor', "move").on("mousedown", function(e) {
                $el.addClass('draggable');
                $drag.addClass('active-handle');
                var z_idx = $el.css('z-index'),
                    drg_h = $el.outerHeight(),
                    drg_w = $el.outerWidth(),
                    pos_y = $el.offset().top + drg_h - e.pageY,
                    pos_x = $el.offset().left + drg_w - e.pageX;
                $el.css('z-index', 1000).parents().on("mousemove", function(e) {
                    $('.gllue-toolbar.draggable').offset({
                        top:e.pageY + pos_y - drg_h,
                        left:e.pageX + pos_x - drg_w
                    }).on("mouseup", function() {
                        $(this).removeClass('draggable').css('z-index', z_idx);
                    }).css('right', 'initial');
                });
                e.preventDefault(); // disable selection
            }).on("mouseup", function() {
                $drag.removeClass('active-handle');
                $el.removeClass('draggable');
            });
        },
        simulateClick: function simulateClick(dom) {
            if (!dom){
                console.error('dispatch empty cb');
                return;
            }
            var event = new MouseEvent('click', {
                'view': window,
                'bubbles': true
            });
            dom.dispatchEvent(event);
        },
        findAndClick: function(clickables, value, getKey){
            if (!getKey){
                getKey = function(elem){
                    return elem.text().trim();
                };
            }
            for (var i = 0; i < clickables.length; ++i){
                var span = $(clickables[i]);
//                console.log(getKey(span));
//                console.log(value);
                if (getKey(span) == value){
                    Utils.simulateClick(clickables[i]);
                    return;
                }
            }
        },

        injectPageExecutor: function(){
            function executor(){
                window.addEventListener("message", function(event) {
                    // We only accept messages from ourselves
                    if (event.source != window) {
                        return;
                    }

                    if (event.data.type && (event.data.type == "GLLUE_EXTENSION")) {
                        var ret;
                        console.log(event.data);
                        if (event.data.script) {
                            ret = eval(event.data.script);
                        } else if (event.data.funcName){
                            var func = window[event.data.funcName];
                            if (func){
                                ret = func.apply(this, event.data.arguments);
                            }
                        }
                        // return code is to distinguish different callee
                        var returnCode = event.data.returnCode;
                        window.postMessage({ type: "FROM_PAGE", returnCode: returnCode, ret: ret }, "*");
                    }
                }, false);
            }

            var pageScriptSrc = "("+executor+"())";
            var script = document.createElement('script');
            script.appendChild(document.createTextNode(pageScriptSrc));
            (document.body || document.head || document.documentElement).appendChild(script);
        },

        // pageFuncCall: call a function in front page with arguments
        pageFuncCall: function(funcName, args){
            return root.Utils.pageFuncHelper({ funcName: funcName, arguments: args});

        },
        // pageEval: eval a script in front page context
        pageEval: function(script, callback){
            return root.Utils.pageFuncHelper({ script: script});
        },

        pageFuncHelper: function(opt){
            var dfd = $.Deferred();
            var randomReturnCode = Math.random().toString(36);
            window.addEventListener("message", function(event){
                if (event.source != window){
                    return;
                }

                if (event.data.type && (event.data.type == "FROM_PAGE") && (event.data.returnCode == randomReturnCode)) {
                    var ret = event.data.ret;
                    dfd.resolve(ret);
                }
            });
            opt.type = 'GLLUE_EXTENSION';
            opt.returnCode = randomReturnCode;
            window.postMessage(opt, "*");
            return dfd;
        },

        afterMilliseconds: function(delay, ret){
            return new Promise(function(resolve, reject){
                setTimeout(function(){
                    resolve(ret)
                }, delay);
            });
        },
        clickProper: function(firstClick, getFinalClick, val, getNext, dataTree, delay){
            var simulateClick = root.Utils.simulateClick;
            var afterMilliseconds = root.Utils.afterMilliseconds;

            if (!delay) {
                delay = 500;
            } else if ((typeof delay) !=='number' ){
                throw "Argument \"Delay\" should be a number in clickProper";
            }

            if ((typeof getNext) !== 'function' || (typeof getFinalClick) !== 'function'){
                throw "Argument \"getProper or getFinalClick\" should be a function";
            }
            var sid = val.sid.toString();
            var tracePath = [sid];
            while (dataTree[sid]){
                sid = dataTree[sid];
                tracePath.unshift(sid);
            }

            simulateClick(firstClick.get(0));

            return afterMilliseconds(delay).then(function(){
                tracePath.forEach(function(sid){
                    var nxt = getNext(sid).get(0);
                    if (nxt){
                        simulateClick(nxt);
                    }
                });
                simulateClick(getFinalClick().get(0));
                return true;
            });

        }
    };

    var Utils = root.Utils;

}(window));

}catch(err){
    Logger.error(err);
}
