/* global Config, Logger, chrome */
try {
    (function WebCallContent(root, $) {
        "use strict";

        if(root[Config.WEBCALL_INDICATOR]) {
            return;
        }

        function uuid() {
            function S4() {
                return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
            }

            // then to call it, plus stitch in '4' in the third group
            return (S4() + S4() + "-" + S4() + "-4" + S4().substr(0, 3) + "-" + S4() + "-" + S4() + S4() + S4()).toLowerCase();
        }


        function getHostByUrl(url) {
            var aTag = document.createElement('a');
            aTag.href = url;
            return aTag.host;
        }

        root[Config.WEBCALL_INDICATOR] = true;

        var SEL_ROOT = '.fn-webcall-popover';
        var SEL_MOBILE = '[name="number"]';
        var SEL_CALL = '.fn-call';
        var SEL_CALL_OUT_LAND = '.fn-outland';
        var SEL_OPTIONS = '[name="options"]';

        var HOST = '//' + getHostByUrl('/');
        var EXT_MSG_URL = HOST + Config.URL.channel_message;
        var USER_CTX_URL = HOST + Config.URL.user_context;
        var $root;
        var $currTarget;
        var CallPrefix = '';
        var outlandPrefix = '0';
        var isCallPrefixLoaded;
        var isLoading;
        var $body = $('body');

        function moveToTarget($el) {
            var position = $el.offset();
            var left = position.left;
            var top = position.top;
            var width = $el.outerWidth();
            var height = $el.outerHeight();
            var $rootWidth = $root.outerWidth();

            $root.css({
                display: 'block',
                top: top + height -1,
                left: left - ($rootWidth - width) / 2
            });

            $root.find(SEL_MOBILE).val($el.text());
        }

        function hidePopover() {
            $root.hide();
        }

        function showPopOver($el) {
            if($root) {
                moveToTarget($el);

                return;
            }

            $root = $body.find(SEL_ROOT);

            var $mobile = $root.find(SEL_MOBILE);
            var $call = $root.find(SEL_CALL);
            var $options = $root.find(SEL_OPTIONS);

            $body.find(SEL_CALL_OUT_LAND).text(outlandPrefix);

            $options.on('change', function optionHandler($e) {
                var sourceMobile = $currTarget.attr('href').replace('tel:', '');

                switch($($e.target).val()) {
                    case 'local':
                        $mobile.val(sourceMobile);
                        break;
                    case 'outer':
                        $mobile.val(outlandPrefix + sourceMobile);
                        break;
                }
            });

            $call.on('click', function() {
                var mobile;
                var option = $root.find('input:checked').val();
                var sourceMobile = $currTarget.attr('href').replace('tel:', '');

                switch(option) {
                    case 'local':
                        mobile = CallPrefix + sourceMobile;
                        break;
                    case 'outer':
                        mobile = CallPrefix + outlandPrefix + sourceMobile;
                        break;
                    case 'custom':
                        mobile = $mobile.val();
                        break;
                }

                var dataset = $currTarget[0].dataset;

                $.ajax({
                    url: EXT_MSG_URL,
                    type: 'post',
                    data: {
                        data: JSON.stringify({
                            type: 'webcall',
                            session_id: uuid(),
                            value: {
                                dialMobile: mobile,
                                mobile: sourceMobile,
                                candidateId: dataset['candidateId'],
                                resumeId: dataset['resumeId'],
                                tabId: window.__tab_id__
                            }
                        })
                    }
                });
            });

            moveToTarget($el);

            $body.on('click.webcall-popver', function ($e) {
                if (!$($e.target).closest($root).length) {
                    hidePopover();
                    return;
                }
            });
        }

        // unsubscribe before register new handler
        $body.off('mouseover.webcall click.webcall');

        function showPopover($e) {
            var $el = $(this);

            if(!root.userContext && !isCallPrefixLoaded && !isLoading) {
                isLoading = true;

                $.getJSON(USER_CTX_URL, function(res) {
                    root.userContext = res;

                    isCallPrefixLoaded = true;
                    CallPrefix = res.config.callprefix || '';
                    outlandPrefix = res.config.outlandPrefix || '0';
                }, function(e) {
                    isLoading = false;
                });
            } else {
                //if($root && !isMouseOverElBoundry($root, {top: -10})){
                if ($root) {
                    $root.css('left', -9999);
                    $root.find(SEL_OPTIONS).eq(0).prop('checked', true);
                }
            }

            $currTarget = $el;

            showPopOver($currTarget);
        }

        // bind events to all tel:XX elements
        function registerPageEvent() {
            //var $tels = $body.find('[href^="tel"]');
            $body.on('mouseover.webcall click.webcall', '[href^="tel"]', showPopover);
        }

        chrome.runtime.sendMessage({
            type: 'phonebox/check'
        }, function (res) {
            if (res) { // phone box client installed
                window.addEventListener('message', function (event) {
                    var data = JSON.parse(event.data)
                    if(data && data.webcallShow === 'webcallShow'){
                        var target = $('[data-woven=\"'+data.target+'\"]').find('[href^="tel"]')[0];
                        showPopover.call(target, target);
                    }
                });
                registerPageEvent();
            }
        });

    })(window, jQuery);

} catch (err) {
    Logger.error(err);
}