/* global chrome */

(function (window, $) {
    'use strict';
    var ctx = chrome.extension.getBackgroundPage();
    var gStorage = ctx.gStorage;
    var utils = ctx.Utils;
    var logo = document.getElementById('logo');
    var urlInput = document.querySelector('#urltext');
    var ap = document.querySelector('#autoupdate');


    var logoTimer;


    function loadLogo(hostUrl, options) {
        var logoUrl = hostUrl.replace(/\/$/, '') + '/logo.png' + '?_=' + (new Date()).getTime();

        var img = document.createElement('img');

        img.onload = function () {
            logo.setAttribute('src', logoUrl);
            logo.classList.remove('hide');

            if (options && options.onSuccess) {
                options.onSuccess();
            }
        };

        img.onerror = function () {
            logo.setAttribute('src', '');
            logo.classList.add('hide');

            if (options && options.onError) {
                options.onError();
            }
        };

        img.src = logoUrl;
    }

    ap.addEventListener('change', function (e) {
        var isUpdate = e.target.checked;

        chrome.storage.sync.set({ 'autoupdate': isUpdate }, function () {
            if (isUpdate) {
                utils.setProxy();
            } else {
                utils.clearProxy();
            }
        });

    }, false);

    gStorage.get('autoupdate', function (items) {
        ap.checked = items['autoupdate'];
    });

    gStorage.getHost().then(function (host) {
        var url = host;
        if (!url) {
            url = 'http://';
        }

        var result = document.querySelector('#opt-result');

        loadLogo(url);

        urlInput.value = url;
        urlInput.addEventListener('input', function (e) {
            var host = this.value.replace(/(http:\/\/.*)\/\s*$/, '$1');

            if (host.indexOf('http') !== 0) {
                host = 'http://' + host;
            }

            // update logo
            clearTimeout(logoTimer);

            logoTimer = setTimeout(function () {
                loadLogo(host, {
                    onSuccess: function () {
                        gStorage.setHost(host).then(function () {

                            result.innerText = "保存成功！";
                            result.classList.remove('hide');

                            setTimeout(function () {
                                chrome.runtime.reload();
                                result.classList.add('hide');
                            }, 1000);

                        });
                    },
                    onError: function () {
                        result.innerText = "无效的URL，请重新设置！";
                        result.classList.remove('hide');
                    }
                });
            }, 1000);

        });
    });
}(window, jQuery));
