/* global chrome, Utils, Logger, $, Config */
try {
    (function(root){
        "use strict";

        var bgCtx = chrome.extension.getBackgroundPage();
        var params = bgCtx.Utils.searchToObject(location.search, true);
        var Config = bgCtx.Config;
        var Utils = bgCtx.Utils;
        var WebCall = bgCtx.WebCall;
        var Session = WebCall.Session;
        var Logger = bgCtx.Logger;
        var Tab = bgCtx.Tab;
        var blurb = chrome.i18n.getMessage;

        var app = angular.module('WebCallApp',['common-module']);
        var $el = document.querySelector('.container');

        var $internalScope;

        window.onbeforeunload = function () {
            if (!$internalScope.idle) {
                return '还在通话中，确定要关闭次窗口吗？';
            }
        };

        window.onunload = window.onclose = function () {
            if (Session.currentSession) {
                Session.currentSession.sessionEnd();
            }
        };

        // util function
        function getServiceByName(name) {
            return angular.element($el).injector().get(name);
        }

        function checkCandidate() {
            if(params.check_mobile && !$internalScope.candidate && !$internalScope.posting) {
                getServiceByName('CandidateService')
                    .getCandidateByNumber(params.check_mobile, function(result) {
                        if (result.candidate) {
                            result.candidate.unique_id = result.candidate.unique_id || 0;
                            $internalScope.candidate = result.candidate;
                        }
                });
            }
        }

        //transform payload data to form data
        app.config(['$httpProvider', function($httpProvider) {
            $httpProvider.defaults.transformRequest = function(obj){
                 var str = [];
                 for(var p in obj){
                   str.push(encodeURIComponent(p) + "=" + encodeURIComponent(obj[p]));
                 }
                 return str.join("&");
            };

            $httpProvider.defaults.headers.post = {
                'Content-Type': 'application/x-www-form-urlencoded'
            };
        }]);

        // register services
        app.factory('CandidateService', ['$http', '$q', function($http, $q) {
            return {
                /*
                * todo: find better solution for nested promise.
                * Please notice this function returned is not a $http request promise,
                * so this is not up to angular standard. This is beacause here have a nest
                * promise, and not found a better solution yet.
                */
                getCandidateByNumber: function(number, cb) {

                    return Config.getAbsUrl('check_exists').then(function(url) {
                        $internalScope.host = bgCtx.Utils.getProtocolAndHostByURL(url);

                        return $http.get(url + number)
                            .then(function(response) {
                                $internalScope.candidateChecked = true;
                                if (response.status === 200 && response.data.status) {
                                    var data = response.data.data;

                                    if(data.candidate[0]) {
                                        data.candidate[0].candidate_id = data.candidate[0].id;
                                        cb({candidate: data.candidate[0]});
                                    }
                                    return;
                                }
                            }, function(response) {
                                // something went wrong
                                return $q.reject(response);
                            });
                    });

                }
            };
        }]);

        // register controllers
        app.controller('WebCallController', ['$rootElement', '$scope', '$q', '$interval', function($rootElement, $scope, $q, $interval) {
            $internalScope = angular.element($el).scope();
            $scope.candidateChecked = false;

            chrome.windows.getCurrent(function (tab) {
                WebCall.popupId = tab.id;
            });

            checkCandidate();

            $scope.action = params.action;
            $scope.number = window.decodeURIComponent(params.tel);
            $scope.idle = true;

            $scope.i18n = {
                refuse: blurb('refuse'),
                answer: blurb('answer'),
                upload: blurb('upload_record'),
                hang_up: blurb('hang_up'),
                add_notes: blurb('add_notes'),
                stop_dial: blurb('stop_dial')
            };

            // DOM EVENTS

            // refuse handler
            $scope.refuse = function($e) {
                $e.preventDefault();

                WebCall.refuseCall();
            };

            // hang up handler
            $scope.stopDial = function($e) {
            };

            $scope.hangUp = function($e) {};

            // answer handler
            $scope.answer = function($e) {
                $e.preventDefault();

            };

            $scope.dial = function ($event) {
                if ($event) {
                    $event.preventDefault();
                }

                $scope.statusMsg = '拨号中...';
                $scope.idle = false;

                // TODO
                new Session();
                Session.currentSession.mobile = $scope.number.replace(/-/g, ',');
                Session.currentSession.dialOut();
            };

            // save record info to server
            $scope.upload = function($event) {
                if ($event){
                    $event.preventDefault();
                }

                $scope.uploading = true;

                Session.lastSession.uploadRecord(function (res) {
                    $scope.$apply(function () {
                        if (res.status) {
                            $scope.statusMsg = '录音上传成功';
                            $scope.canUpload = false;
                        } else {
                            $scope.statusMsg = '录音上传失败';
                        }

                        $scope.uploading = false;
                    });

                });

            };

            $scope.tab_id = params.tab_id;

            var start;
            var timer;
            $scope.startTimer = function () {
                start = Date.now();
                timer = $interval(function () {
                    $scope.time = Utils.formatDate.call(new Date(new Date() - start), 'mm分ss秒');
                }, 1000);
            };

            $scope.stopTimer = function () {
                $interval.cancel(timer);

                $scope.time = null;
                $scope.duration = null;
            };

            // notify web page to open note modal
            $scope.addNote = function($e) {
                $e.preventDefault();
                var candidate = $scope.candidate;
                var tabId = parseInt($scope.tab_id, 10);
                var uuid;
                var params = {
                    _modal_name_: 'add-note',
                    externalType: 'candidate',
                    optionType: 'candidate_note_category',
                    id: candidate.id,
                    target: [candidate.englishName, candidate.chineseName].join(' ').trim()
                };

                if (Session.currentSession) {
                    if (Session.lastSession) {
                        uuid = Session.lastSession.uuid;
                    } else {
                        uuid = Session.currentSession.uuid;
                    }
                } else if (Session.lastSession) {
                    uuid = Session.lastSession.uuid;
                }

                if (uuid) {
                    params['call_session_uuid'] = uuid;
                }

                Tab.sendMsgToTab(tabId,
                    {
                        type: 'hub-msg',
                        value: JSON.stringify({
                            topic: 'large-modal/show',
                            params: params,
                            // when target tab is closed should create a new tab with use this url
                            url: $scope.host + '/crm#candidate/detail?id=' + candidate.id + '&modal=add-note&call_session_uuid='+ uuid
                        })
                    },
                    function(error) {
                        if(error) {
                            Logger.error(error);

                            return;
                        }

                        Tab.setTabToActive(tabId);

                    });
            };

            $rootElement.children().removeClass('hide');

            if ($scope.action === 'dial_out') {
                $scope.statusMsg = '拨出';
                $scope.idle = false;
                $scope.dial();
            } else if ($scope.action === 'call_in') {
                $scope.statusMsg = '来电';
                $scope.idle = false;
            }

            WebCall.registerEventHandler('onPhoneHook', function (data) {
                if (Session.currentSession && Session.currentSession.status === 'call_in') {
                    $scope.stopTimer();
                    $scope.startTimer();
                    $scope.statusMsg = '通话中';
                    $scope.idle = false;

                    if (Session.currentSession) {
                        Session.currentSession.candidateId = $scope.candidate ? $scope.candidate.id : null;
                    }
                }
            });

            WebCall.registerEventHandler('onPhoneHang', function (data) {
                $scope.stopTimer();
                $scope.statusMsg = '通话结束';
                $scope.idle = true;

                if (Session.lastSession && Session.lastSession.recordFile && Session.lastSession.candidateId) {
                    $scope.canUpload = true;
                }
            });

            WebCall.registerEventHandler('onRemoteHook', function (data) {
                $scope.stopTimer();
                $scope.startTimer();
                $scope.statusMsg = '通话中';
                $scope.idle = false;

                if (Session.currentSession) {
                    Session.currentSession.candidateId = $scope.candidate.id;
                }
            });

            WebCall.registerEventHandler('onRemoteHang', function (data) {
                $scope.stopTimer();
                $scope.statusMsg = '对方已挂机';
                $scope.idle = true;

                if (Session.lastSession && Session.lastSession.recordFile && Session.lastSession.candidateId) {
                    $scope.canUpload = true;
                }
            });

            WebCall.registerEventHandler('onBusy', function (data) {
                $scope.statusMsg = '对方占线中';
                $scope.idle = true;
            });

            WebCall.registerEventHandler('onRefuseEnd', function (data) {
                $scope.statusMsg = '未接通';
                $scope.idle = true;
            });

            WebCall.registerEventHandler('onDoStartDial', function (data) {
                $scope.statusMsg = '拨号中...';
                $scope.idle = false;
            });

            WebCall.registerEventHandler('onPSTNFree', function (data) {
                $scope.statusMsg = '';
                $scope.idle = true;
            });
        }]);

        angular.bootstrap(window.document.body, ['WebCallApp']);
    })(window);

} catch (err) {
    Logger.error(err);
}