window.loaderGllueWebcall =  function () {
/* global chrome */

(function(root) {
    "use strict";

    var HOST_NAME = 'gllueHost';
    chrome.extension.onConnect.addListener(function (port) {

        chrome.storage.sync.get(HOST_NAME, function (items) {
            root.postMessage({
                type: 'injected',
                value: {
                    version: chrome.runtime.getManifest().version,
                    host: items[HOST_NAME]
                }
            }, '*');
        });

        port.onMessage.addListener(function (message, port) {
            var portName = port.name;

            if (portName === 'gllue-ext') {
                console.log("message from extension: ", message);

                root.postMessage(message, '*');
            }
        });
    });
}(window));

(function(root){
    root.ExposedMethod = {
        sendResult: function(msg){
            msg.type = 'extensionResult';
            root.postMessage(msg, "*");
            return Promise.resolve();
        },

        changePostingModalJoborderId: function(joborderId){
            Position.setJoborderId(joborderId);
            return Promise.resolve();
        },
        getPositionInfo: function(){
            if (!root.Position){
                return Promise.resolve();
            }
            return new Promise(function(resolve, reject){
                setTimeout(function(){
                    var positionInfo = root.Position.getPositionInfo();
                    resolve(positionInfo);
                }, 1000);
            });
        },
        webRefreshJoborderList: function(){
            var msg = {};
            msg.type = 'refreshPositionList';
            root.postMessage(msg, "*");
            return Promise.resolve();
        }
    }


})(window);
(function(root){
    var ExposedMethod = root.ExposedMethod;
    function emptyFunction(){}
    chrome.runtime.onMessage.addListener(function(request, sender, sendResponse){
        if (request.name === 'gllue-ext-function-call'){
            // 对于publish的方法,我们没有必要sendResponse,不然会报错
            if (request.publish){
                sendResponse = emptyFunction;
            }
            var msg = request.msg;
            var methodName = msg.methodName;

            console.log("[gllue extension function call]");
            console.log("methodName:"+methodName);
            console.log("args:"+msg.args);
            var args = JSON.parse(msg.args);
            var func = ExposedMethod[methodName];
            func.apply(ExposedMethod, args).then(function(res){
                var response = {
                    ack: true,
                    ret: res
                };
                sendResponse(response);
            }, function(err){
                sendResponse({
                    ack: true,
                    ret: err
                });
            });
            return true;
        }
    });

})(window);
/* global chrome, Config */

(function(root) {
    'use strict';

    if(root.__gllueposting__) {
        return;
    }

    root.__gllueposting__ = true;

    document.querySelector('html').classList.add('gllue-ext-loaded');
    document.querySelector('html').classList.remove('gllue-ext-unload');

    // notify extension the page is going to refresh
    root.onbeforeunload = function() {
        console.log("On before unload content script.");

        // TODO: check which function to subscribe this
        chrome.runtime.sendMessage({
            type: 'refresh'
        });

        // TODO: support old version
        chrome.runtime.sendMessage({
            type: 'page-refresh'
        });

        return;
    };

    chrome.extension.onRequest.addListener(
        function(request, sender, sendResponse) {
            if(request.type === "getVariable"){
                sendResponse({value: root[request.value], type: "getVariable"});
            }
        }
    );
}(window));

/* global Config, Logger, chrome */
try {
    (function WebCallContent(root, $) {
        "use strict";

        if(root[Config.WEBCALL_INDICATOR]) {
            return;
        }

        function uuid() {
            function S4() {
                return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
            }

            // then to call it, plus stitch in '4' in the third group
            return (S4() + S4() + "-" + S4() + "-4" + S4().substr(0, 3) + "-" + S4() + "-" + S4() + S4() + S4()).toLowerCase();
        }


        function getHostByUrl(url) {
            var aTag = document.createElement('a');
            aTag.href = url;
            return aTag.host;
        }

        root[Config.WEBCALL_INDICATOR] = true;

        var SEL_ROOT = '.fn-webcall-popover';
        var SEL_MOBILE = '[name="number"]';
        var SEL_CALL = '.fn-call';
        var SEL_CALL_OUT_LAND = '.fn-outland';
        var SEL_OPTIONS = '[name="options"]';

        var HOST = '//' + getHostByUrl('/');
        var EXT_MSG_URL = HOST + Config.URL.channel_message;
        var USER_CTX_URL = HOST + Config.URL.user_context;
        var $root;
        var $currTarget;
        var CallPrefix = '';
        var outlandPrefix = '0';
        var isCallPrefixLoaded;
        var isLoading;
        var $body = $('body');

        function moveToTarget($el) {
            var position = $el.offset();
            var left = position.left;
            var top = position.top;
            var width = $el.outerWidth();
            var height = $el.outerHeight();
            var $rootWidth = $root.outerWidth();

            $root.css({
                display: 'block',
                top: top + height -1,
                left: left - ($rootWidth - width) / 2
            });

            $root.find(SEL_MOBILE).val($el.text());
        }

        function hidePopover() {
            $root.hide();
        }

        function showPopOver($el) {
            if($root) {
                moveToTarget($el);

                return;
            }

            $root = $body.find(SEL_ROOT);

            var $mobile = $root.find(SEL_MOBILE);
            var $call = $root.find(SEL_CALL);
            var $options = $root.find(SEL_OPTIONS);

            $body.find(SEL_CALL_OUT_LAND).text(outlandPrefix);

            $options.on('change', function optionHandler($e) {
                var sourceMobile = $currTarget.attr('href').replace('tel:', '');

                switch($($e.target).val()) {
                    case 'local':
                        $mobile.val(sourceMobile);
                        break;
                    case 'outer':
                        $mobile.val(outlandPrefix + sourceMobile);
                        break;
                }
            });

            $call.on('click', function() {
                var mobile;
                var option = $root.find('input:checked').val();
                var sourceMobile = $currTarget.attr('href').replace('tel:', '');

                switch(option) {
                    case 'local':
                        mobile = CallPrefix + sourceMobile;
                        break;
                    case 'outer':
                        mobile = CallPrefix + outlandPrefix + sourceMobile;
                        break;
                    case 'custom':
                        mobile = $mobile.val();
                        break;
                }

                var dataset = $currTarget[0].dataset;

                $.ajax({
                    url: EXT_MSG_URL,
                    type: 'post',
                    data: {
                        data: JSON.stringify({
                            type: 'webcall',
                            session_id: uuid(),
                            value: {
                                dialMobile: mobile,
                                mobile: sourceMobile,
                                candidateId: dataset['candidateId'],
                                resumeId: dataset['resumeId'],
                                tabId: window.__tab_id__
                            }
                        })
                    }
                });
            });

            moveToTarget($el);

            $body.on('click.webcall-popver', function ($e) {
                if (!$($e.target).closest($root).length) {
                    hidePopover();
                    return;
                }
            });
        }

        // unsubscribe before register new handler
        $body.off('mouseover.webcall click.webcall');

        function showPopover($e) {
            var $el = $(this);

            if(!root.userContext && !isCallPrefixLoaded && !isLoading) {
                isLoading = true;

                $.getJSON(USER_CTX_URL, function(res) {
                    root.userContext = res;

                    isCallPrefixLoaded = true;
                    CallPrefix = res.config.callprefix || '';
                    outlandPrefix = res.config.outlandPrefix || '0';
                }, function(e) {
                    isLoading = false;
                });
            } else {
                //if($root && !isMouseOverElBoundry($root, {top: -10})){
                if ($root) {
                    $root.css('left', -9999);
                    $root.find(SEL_OPTIONS).eq(0).prop('checked', true);
                }
            }

            $currTarget = $el;

            showPopOver($currTarget);
        }

        // bind events to all tel:XX elements
        function registerPageEvent() {
            //var $tels = $body.find('[href^="tel"]');
            $body.on('mouseover.webcall click.webcall', '[href^="tel"]', showPopover);
        }

        chrome.runtime.sendMessage({
            type: 'phonebox/check'
        }, function (res) {
            if (res) { // phone box client installed
                window.addEventListener('message', function (event) {
                    var data = JSON.parse(event.data)
                    if(data && data.webcallShow === 'webcallShow'){
                        var target = $('[data-woven=\"'+data.target+'\"]').find('[href^="tel"]')[0];
                        showPopover.call(target, target);
                    }
                });
                registerPageEvent();
            }
        });

    })(window, jQuery);

} catch (err) {
    Logger.error(err);
}

}