/* global Config, chrome, Storage, DEBUG, DEBUG_SAAS */

/*
 * All config related logics.
 */
(function(root) {
    'use strict';

    // if already loaded once, then do nothing
    if(root.Config) {
        return;
    }

    if (typeof DEBUG === 'undefined') {
        window.DEBUG = false;
    }

    if (typeof DEBUG_SAAS === 'undefined') {
        window.DEBUG_SAAS = false;
    }

    var UNDEF;
    var CONFIG = root.Config = {
        debug: DEBUG,

        HUB_HOST: (DEBUG || DEBUG_SAAS) ? 'http://localhost:8005' : 'https://chrome-extension.gllue.me',
        SAAS_CONTENT_LOADER_PATH: '/js/content/loader.js',
        SAAS_BACKGROUND_LOADER_PATH: '/js/background/loader.js',
        SAAS_OPTIONS_LOADER_PATH: '/js/options/loader.js',

        // storage key of gllue host
        HOST_NAME : 'gllueHost',

        HOST: '',

        REG_URL: /ΩΩ/,

        NATIVE_CLIENT_HOST: "com.gllue.phonebox",

        TITLE: 'Posting Message',

        // Indicator used to check content script is loaded or not
        INDICATOR: '__gllueposting__',

        EXT_INDICATOR: '__gllueExtractorLoaded__',

        // Indicator used to check webcall content script
        WEBCALL_INDICATOR: '__webcall__',

        // All Domain need to check
        DOMAIN: ['LIEPIN', 'ZHAOPIN', '51JOB', 'LINKEDIN'],
        // Cookie check list
        DOMAIN_REQUIRED_COOKIES: {
            'LIEPIN': {
                lt_auth: UNDEF,
                user_id: UNDEF,
                user_kind: UNDEF,
                user_login: UNDEF,
                user_name: UNDEF
            },
            'ZHAOPIN': {
                RDsUserInfo: UNDEF
            },
            '51JOB': {
                HRUSERINFO: UNDEF
            }
        },
        // All platform login state
        DOMAIN_MISC: {
            'LIEPIN': {
                url: 'http://liepin.com',
                domain: 'liepin.com',
                isLogin: false
            },
            'ZHAOPIN': {
                url: 'http://zhaopin.com',
                domain: 'zhaopin.com',
                isLogin: false
            },
            '51JOB': {
                url: 'http://51job.com',
                domain: '51job.com',
                isLogin: false
            },
            'LINKEDIN': {
                url: 'http://www.linkedin.com',
                domain: 'linkedin.com',
                isLogin: false
            }
        },

        ERROR: {
            NOT_IN_GLLUE: "Current tab is not running gllue app."
        },

        URL: {
            account: "/rest/jobposting/account/list?no_count",
            session_status: "/rest/jobposting/session/status",
            session_suspend: "/rest/jobposting/session/suspend?session_id=",
            session_resume: "/rest/jobposting/session/resume?session_id=",
            channel_setting: "/channel/setting",
            login: "/rest/jobposting/login?site=",
            channel_login: "templates/popup/login.html",
            webcall: "webcall-popup.html",
            check_exists: "/rest/candidate/check_exists?mobile=",
            addcall: "/rest/phonebox/add",
            call_history: "/rest/phonebox/list?number=",
            log: '/ext/log',
            channel_message: '/rest/jobposting/extension/message?',
            user_context: '/rest/user/context',
            posting_queue: '/rest/jobposting/queue',
            joborder: '/rest/joborder/livejobs?unlimited',
            convert_form: '/rest/jobposting/form/<joborder>/convert_to_site?to=<site>',
            position_save: '/rest/jobposting/posting_add_from_ext',
            position_check: '/rest/jobposting/get_info_from_ext',
            add_to_job: '/rest/jobsubmission/add',
            add_note: '/rest/note/add',
            extension_config: '/extension_config'
        },

        //WEB_CALL_TOPICS: {
        //    CALL_BEFORE: 'webcall/before',
        //    CALL_OUT: 'webcall/out',
        //    CALL_IN: 'webcall/in',
        //    CALL_IN_NUMBER: 'webcall/in-number',
        //    CALL_INERRUPT: 'webcall/interrupt',
        //    CALL_IN_REFUSE: 'webcall/refuse',
        //    CALL_END: 'webcall/end',
        //    CALL_MISSED: 'webcall/missed',
        //    PHONE_HOOK: 'webcall/hook',
        //    REMOTE_HOOK: 'webcall/remote-hook'
        //},


        WebPageIndicator: 'jobposting/extension/message',

        userAgent: window.navigator.userAgent,
        REVISION_ID: 'unknonw',
        EXT_VERSION: chrome.runtime.getManifest().version,

        getDomainByURL: function(url) {
            var domain;

            CONFIG.DOMAIN.forEach(function(d) {
                if (url.indexOf(d.toLowerCase()) > -1) {
                    domain = CONFIG.DOMAIN_MISC[d].url.replace(/http(s)?:\/\//g, "");

                    return false;
                }
            });

            return domain;
        },

        getSiteNameByURL: function(url) {
            var result = "";

            CONFIG.DOMAIN.forEach(function(d) {
                if (url.indexOf(d.toLowerCase()) > -1) {
                    result = chrome.i18n.getMessage(d.toLowerCase());

                    return false;
                }
            });

            return result;
        },

        getAbsUrl: function(urlName) {

            function notify(dfd) {
                dfd.resolve((CONFIG.HOST || 'http://demo.gllue.com') + CONFIG.URL[urlName]);
            }

            return $.Deferred(function (dfd) {
                if (CONFIG.HOST) {
                    notify(dfd);
                } else {
                    chrome.storage.sync.get(CONFIG.HOST_NAME, function (obj) {
                        CONFIG.HOST = obj[CONFIG.HOST_NAME];
                        notify(dfd);
                    });
                }
            });
        }
    };

    // initial get host name
    chrome.storage.sync.get(CONFIG.HOST_NAME, function (obj) {
        CONFIG.HOST = obj[CONFIG.HOST_NAME];
    });

})(this);
