/* global chrome, Config */
/*
 *  In case of computing all log info, such as save to DB etc.
 */
(function(root) {
    'use strict';

    var Logger = root.Logger = {};


    ['log', 'info', 'warn', 'error', 'debug', 'assert', 'group', 'dir', 'groupEnd', 'trace'].forEach(function(option) {
        root.Logger[option] = function() {
            console[option].apply(console, arguments);

            if(option === 'error') {
                var err = new Error();
                if (!chrome.windows) {
                    // if in chrome content script
                    // chrome.runtime.sendMessage({
                    //     type: 'errorHandling',
                    //     data: {
                    //         msg: JSON.stringify(arguments),
                    //         stack: err.stack
                    //     }
                    // }, function(response) {
                    // });
                } else {
                    Logger.store(JSON.stringify(arguments), err.stack);
                }
            }
        };
    });

    Logger.post = function post(url, data) {
        // if in chrome background
        $.ajax({
            url: url,
            data: {
                data: JSON.stringify(data)
            },
            type: 'post'
        }).fail(function() {
            Logger.warn('Error Log send failed!!!');
        });
    };

    Logger.store = function store(msg, stack, obj, page_url) {
        var data = {
            type: 'error',
            host: Config.host,
            user_id: ((root.userContext && root.userContext.user) ? root.userContext.user.id : null),
            ext_version: Config.EXT_VERSION,
            revision_id: Config.REVISION_ID,
            user_agent: Config.userAgent,
            message: msg,
            stack: stack,
            page_url: page_url || window.location.href,
            extra: obj
        };

        // Config.getAbsUrl('log').then(function(url) {
            // Logger.post(url, data);
        // });
    };

    Logger.onerror = function (msg, url, line, column, errorObj) {
        if (!chrome.windows) {
            // if in chrome content script
            return chrome.runtime.sendMessage({
                type: 'errorHandling',
                data: {
                    msg: msg,
                    page_url: window.location.href,
                    url: url,
                    line: line,
                    column: column,
                    stack: (errorObj ? errorObj.stack : '')
                }
            }, function(response) {
            });
        } else {
            return Logger.store(msg, (errorObj ? errorObj.stack : ''), {
                jsfile: url,
                line: line
            });
        }
    };

    // Error Log Handler
    window.onerror =  function (/*msg, url, line, column, errorObj*/) {
        return Logger.onerror.apply(root, Array.prototype.slice.call(arguments));
    };

})(this);
